@@ -1,6 +1,6 @@
 /* This is dvipdfmx, an eXtended version of dvipdfm by Mark A. Wicks.
 
-    Copyright (C) 2002-2014 by Jin-Hwan Cho and Shunsaku Hirata,
+    Copyright (C) 2002-2016 by Jin-Hwan Cho and Shunsaku Hirata,
     the dvipdfmx project team.
     
     Copyright (C) 1998, 1999 by Mark A. Wicks <mwicks@kettering.edu>
@@ -32,9 +32,10 @@
 #define pdfbox_trim  4
 #define pdfbox_art   5
 
-extern int pdf_copy_clip (FILE *image_file, int page_index, double x_user, double y_user);
+extern int pdf_copy_clip (FILE *image_file, int page_index,
+                          double x_user, double y_user);
 
-extern int pdf_include_page (pdf_ximage *ximage, FILE *file,
-			     const char *filename);
+extern int pdf_include_page (pdf_ximage * ximage, FILE *file,
+                             const char  *ident, load_options options);
 
 #endif /* _EPDF_H_ */
