@@ -1,6 +1,6 @@
 /* This is dvipdfmx, an eXtended version of dvipdfm by Mark A. Wicks.
 
-    Copyright (C) 2008-2015 by Jin-Hwan Cho, Matthias Franz, and Shunsaku Hirata,
+    Copyright (C) 2008-2016 by Jin-Hwan Cho, Matthias Franz, and Shunsaku Hirata,
     the dvipdfmx project team.
     
     Copyright (C) 1998, 1999 by Mark A. Wicks <mwicks@kettering.edu>
@@ -88,6 +88,7 @@ read_thumbnail (const char *thumb_filename)
   pdf_obj *image_ref;
   int      xobj_id;
   FILE    *fp;
+  load_options options = {1, 0, NULL};
 
   fp = MFOPEN(thumb_filename, FOPEN_RBIN_MODE);
   if (!fp) {
@@ -101,7 +102,7 @@ read_thumbnail (const char *thumb_filename)
   }
   MFCLOSE(fp);
 
-  xobj_id = pdf_ximage_findresource(thumb_filename, 0, NULL);
+  xobj_id = pdf_ximage_findresource(thumb_filename, options);
   if (xobj_id < 0) {
     WARN("Could not read thumbnail file \"%s\".", thumb_filename);
     image_ref = NULL;
@@ -179,7 +180,7 @@ typedef struct pdf_olitem
 typedef struct pdf_bead
 {
   char    *id;
-  long     page_no;
+  int      page_no;
   pdf_rect rect;
 } pdf_bead;
 
@@ -187,8 +188,8 @@ typedef struct pdf_article
 {
   char     *id;
   pdf_obj  *info;
-  long      num_beads;
-  long      max_beads;
+  int       num_beads;
+  int       max_beads;
   pdf_bead *beads;
 } pdf_article;
 
@@ -217,8 +218,8 @@ typedef struct pdf_doc
     pdf_rect mediabox;
     pdf_obj *bop, *eop;
 
-    long      num_entries; /* This is not actually total number of pages. */
-    long      max_entries;
+    int       num_entries; /* This is not actually total number of pages. */
+    int       max_entries;
     pdf_page *entries;
   } pages;
 
@@ -229,8 +230,8 @@ typedef struct pdf_doc
   } outlines;
 
   struct {
-    long         num_entries;
-    long         max_entries;
+    int          num_entries;
+    int          max_entries;
     pdf_article *entries;
   } articles;
 
@@ -322,10 +323,10 @@ pdf_doc_close_catalog (pdf_doc *p)
 #define MAXPAGES(p)  (p->pages.max_entries)
 
 static void
-doc_resize_page_entries (pdf_doc *p, long size)
+doc_resize_page_entries (pdf_doc *p, int size)
 {
   if (size > MAXPAGES(p)) {
-    long i;
+    int i;
 
     p->pages.entries = RENEW(p->pages.entries, size, struct pdf_page);
     for (i = p->pages.max_entries; i < size; i++) {
@@ -349,7 +350,7 @@ doc_resize_page_entries (pdf_doc *p, long size)
 }
 
 static pdf_page *
-doc_get_page_entry (pdf_doc *p, unsigned long page_no)
+doc_get_page_entry (pdf_doc *p, unsigned int page_no)
 {
   pdf_page *page;
 
@@ -433,17 +434,23 @@ pdf_doc_set_eop_content (const char *content, unsigned length)
    systems that do not support the tm_gmtoff in struct tm,
    or have a timezone variable.  Such as i386-solaris.  */
 
-static long
+static int32_t
 compute_timezone_offset()
 {
-  const time_t now = time(NULL);
+  time_t now;
   struct tm tm;
   struct tm local;
   time_t gmtoff;
 
-  localtime_r(&now, &local);
-  gmtime_r(&now, &tm);
-  return (mktime(&local) - mktime(&tm));
+  now = get_unique_time_if_given();
+  if (now == INVALID_EPOCH_VALUE) {
+    now = time(NULL);
+    localtime_r(&now, &local);
+    gmtime_r(&now, &tm);
+    return (mktime(&local) - mktime(&tm));
+  } else {
+    return(0);
+  }
 }
 
 #endif /* HAVE_TIMEZONE */
@@ -452,31 +459,36 @@ compute_timezone_offset()
 /*
  * Docinfo
  */
-static long
+static int
 asn_date (char *date_string)
 {
-  long        tz_offset;
+  int32_t     tz_offset;
   time_t      current_time;
   struct tm  *bd_time;
 
-  time(&current_time);
-  bd_time = localtime(&current_time);
+  current_time = get_unique_time_if_given();
+  if (current_time == INVALID_EPOCH_VALUE) {
+    time(&current_time);
+    bd_time = localtime(&current_time);
 
 #ifdef HAVE_TM_GMTOFF
-  tz_offset = bd_time->tm_gmtoff;
+    tz_offset = bd_time->tm_gmtoff;
 #else
 #  ifdef HAVE_TIMEZONE
-  tz_offset = -timezone;
+    tz_offset = -timezone;
 #  else
-  tz_offset = compute_timezone_offset();
+    tz_offset = compute_timezone_offset();
 #  endif /* HAVE_TIMEZONE */
 #endif /* HAVE_TM_GMTOFF */
-
-  sprintf(date_string, "D:%04d%02d%02d%02d%02d%02d%c%02ld'%02ld'",
-	  bd_time->tm_year + 1900, bd_time->tm_mon + 1, bd_time->tm_mday,
-	  bd_time->tm_hour, bd_time->tm_min, bd_time->tm_sec,
-	  (tz_offset > 0) ? '+' : '-', labs(tz_offset) / 3600,
-                                      (labs(tz_offset) / 60) % 60);
+  } else {
+    bd_time = gmtime(&current_time);
+    tz_offset = 0;
+  }
+  sprintf(date_string, "D:%04d%02d%02d%02d%02d%02d%c%02d'%02d'",
+          bd_time->tm_year + 1900, bd_time->tm_mon + 1, bd_time->tm_mday,
+          bd_time->tm_hour, bd_time->tm_min, bd_time->tm_sec,
+          (tz_offset > 0) ? '+' : '-', abs(tz_offset) / 3600,
+                                      (abs(tz_offset) / 60) % 60);
 
   return strlen(date_string);
 }
@@ -605,7 +617,7 @@ pdf_doc_add_page_resource (const char *category,
   resources = pdf_doc_get_page_resources(p, category);
   duplicate = pdf_lookup_dict(resources, resource_name);
   if (duplicate && pdf_compare_reference(duplicate, resource_ref)) {
-    WARN("Conflicting page resource found (page: %ld, category: %s, name: %s).",
+    WARN("Conflicting page resource found (page: %d, category: %s, name: %s).",
          pdf_doc_current_page_number(), category, resource_name);
     WARN("Ignoring...");
     pdf_release_obj(resource_ref);
@@ -712,11 +724,11 @@ doc_flush_page (pdf_doc *p, pdf_page *page, pdf_obj *parent_ref)
 #define PAGE_CLUSTER 4
 static pdf_obj *
 build_page_tree (pdf_doc  *p,
-                 pdf_page *firstpage, long num_pages,
+                 pdf_page *firstpage, int num_pages,
                  pdf_obj  *parent_ref)
 {
   pdf_obj *self, *self_ref, *kids;
-  long     i;
+  int      i;
 
   self = pdf_new_dict();
   /*
@@ -746,7 +758,7 @@ build_page_tree (pdf_doc  *p,
     }
   } else if (num_pages > 0) {
     for (i = 0; i < PAGE_CLUSTER; i++) {
-      long start, end;
+      int start, end;
 
       start = (i*num_pages)/PAGE_CLUSTER;
       end   = ((i+1)*num_pages)/PAGE_CLUSTER;
@@ -804,7 +816,7 @@ pdf_doc_close_page_tree (pdf_doc *p)
 {
   pdf_obj *page_tree_root;
   pdf_obj *mediabox;
-  long     page_no;
+  int      page_no;
 
   /*
    * Do consistency check on forward references to pages.
@@ -880,6 +892,35 @@ pdf_doc_close_page_tree (pdf_doc *p)
   return;
 }
 
+int
+pdf_doc_get_page_count (pdf_file *pf)
+{
+  int      count = 0;
+  pdf_obj *page_tree = NULL;
+  pdf_obj *catalog;
+
+  catalog = pdf_file_get_catalog(pf);
+
+  page_tree = pdf_deref_obj(pdf_lookup_dict(catalog, "Pages"));
+
+  if (!PDF_OBJ_DICTTYPE(page_tree)) {
+    return 0;
+  }
+
+  {
+    pdf_obj *tmp = pdf_deref_obj(pdf_lookup_dict(page_tree, "Count"));
+    if (!PDF_OBJ_NUMBERTYPE(tmp)) {
+      if (tmp)
+        pdf_release_obj(tmp);
+      return 0;
+    }
+    count = pdf_number_value(tmp);
+    pdf_release_obj(tmp);
+  }
+
+  return count;
+}
+
 /*
  * From PDFReference15_v6.pdf (p.119 and p.834)
  *
@@ -932,11 +973,16 @@ pdf_doc_close_page_tree (pdf_doc *p)
  * displayed or printed. The value must be a multiple of 90. Default value: 0.
  */
 
+/* count_p removed: Please use different interface if you want to get total page
+ * number. pdf_doc_get_page() is obviously not an interface to do such.
+ */
 pdf_obj *
-pdf_doc_get_page (pdf_file *pf, long page_no, long *count_p,
-		  pdf_rect *bbox, pdf_obj **resources_p) {
+pdf_doc_get_page (pdf_file *pf,
+                  int page_no, int options,             /* load options */
+                  pdf_rect *bbox, pdf_obj **resources_p /* returned values */
+                  ) {
   pdf_obj *page_tree = NULL;
-  pdf_obj *resources = NULL, *box = NULL, *rotate = NULL;
+  pdf_obj *resources = NULL, *box = NULL, *rotate = NULL, *medbox = NULL;
   pdf_obj *catalog;
 
   catalog = pdf_file_get_catalog(pf);
@@ -947,21 +993,19 @@ pdf_doc_get_page (pdf_file *pf, long page_no, long *count_p,
     goto error;
 
   {
-    long count;
+    int count;
     pdf_obj *tmp = pdf_deref_obj(pdf_lookup_dict(page_tree, "Count"));
     if (!PDF_OBJ_NUMBERTYPE(tmp)) {
       if (tmp)
-	pdf_release_obj(tmp);
+        pdf_release_obj(tmp);
       goto error;
     }
     count = pdf_number_value(tmp);
     pdf_release_obj(tmp);
-    if (count_p)
-      *count_p = count;
     if (page_no <= 0 || page_no > count) {
-	WARN("Page %ld does not exist.", page_no);
-	goto error_silent;
-      }
+      WARN("Page %ld does not exist.", page_no);
+      goto error_silent;
+    }
   }
 
   /*
@@ -969,69 +1013,88 @@ pdf_doc_get_page (pdf_file *pf, long page_no, long *count_p,
    * (Note that these entries can be inherited.)
    */
   {
+    pdf_obj *art_box = NULL, *trim_box = NULL, *bleed_box = NULL;
     pdf_obj *media_box = NULL, *crop_box = NULL, *kids, *tmp;
     int depth = PDF_OBJ_MAX_DEPTH;
-    long page_idx = page_no-1, kids_length = 1, i = 0;
+    int page_idx = page_no-1, kids_length = 1, i = 0;
 
     while (--depth && i != kids_length) {
       if ((tmp = pdf_deref_obj(pdf_lookup_dict(page_tree, "MediaBox")))) {
-	if (media_box)
-	  pdf_release_obj(media_box);
-	media_box = tmp;
+        if (media_box)
+          pdf_release_obj(media_box);
+        media_box = tmp;
       }
 
       if ((tmp = pdf_deref_obj(pdf_lookup_dict(page_tree, "CropBox")))) {
-	if (crop_box)
-	  pdf_release_obj(crop_box);
-	crop_box = tmp;
+        if (crop_box)
+          pdf_release_obj(crop_box);
+        crop_box = tmp;
+      }
+
+      if ((tmp = pdf_deref_obj(pdf_lookup_dict(page_tree, "ArtBox")))) {
+        if (art_box)
+          pdf_release_obj(art_box);
+        art_box = tmp;
+      }
+
+      if ((tmp = pdf_deref_obj(pdf_lookup_dict(page_tree, "TrimBox")))) {
+        if (trim_box)
+          pdf_release_obj(trim_box);
+        trim_box = tmp;
+      }
+
+      if ((tmp = pdf_deref_obj(pdf_lookup_dict(page_tree, "BleedBox")))) {
+        if (bleed_box)
+          pdf_release_obj(bleed_box);
+        bleed_box = tmp;
       }
 
       if ((tmp = pdf_deref_obj(pdf_lookup_dict(page_tree, "Rotate")))) {
-	if (rotate)
-	  pdf_release_obj(rotate);
-	rotate = tmp;
+        if (rotate)
+          pdf_release_obj(rotate);
+        rotate = tmp;
       }
 
       if ((tmp = pdf_deref_obj(pdf_lookup_dict(page_tree, "Resources")))) {
-	if (resources)
-	  pdf_release_obj(resources);
-	resources = tmp;
+        if (resources)
+          pdf_release_obj(resources);
+        resources = tmp;
       }
 
       kids = pdf_deref_obj(pdf_lookup_dict(page_tree, "Kids"));
       if (!kids)
-	break;
+        break;
       else if (!PDF_OBJ_ARRAYTYPE(kids)) {
-	pdf_release_obj(kids);
-	goto error;
+        pdf_release_obj(kids);
+        goto error;
       }
       kids_length = pdf_array_length(kids);
 
       for (i = 0; i < kids_length; i++) {
-	long count;
+        int count;
 
-	pdf_release_obj(page_tree);
-	page_tree = pdf_deref_obj(pdf_get_array(kids, i));
-	if (!PDF_OBJ_DICTTYPE(page_tree))
-	  goto error;
+        pdf_release_obj(page_tree);
+        page_tree = pdf_deref_obj(pdf_get_array(kids, i));
+        if (!PDF_OBJ_DICTTYPE(page_tree))
+          goto error;
 
-	tmp = pdf_deref_obj(pdf_lookup_dict(page_tree, "Count"));
-	if (PDF_OBJ_NUMBERTYPE(tmp)) {
-	  /* Pages object */
-	  count = pdf_number_value(tmp);
-	  pdf_release_obj(tmp);
-	} else if (!tmp)
-	  /* Page object */
-	  count = 1;
-	else {
-	  pdf_release_obj(tmp);
-	  goto error;
-	}
+        tmp = pdf_deref_obj(pdf_lookup_dict(page_tree, "Count"));
+        if (PDF_OBJ_NUMBERTYPE(tmp)) {
+          /* Pages object */
+          count = pdf_number_value(tmp);
+          pdf_release_obj(tmp);
+        } else if (!tmp)
+          /* Page object */
+          count = 1;
+        else {
+          pdf_release_obj(tmp);
+          goto error;
+        }
 
-	if (page_idx < count)
-	  break;
+        if (page_idx < count)
+          break;
 
-	page_idx -= count;
+        page_idx -= count;
       }
       
       pdf_release_obj(kids);
@@ -1039,24 +1102,87 @@ pdf_doc_get_page (pdf_file *pf, long page_no, long *count_p,
 
     if (!depth || kids_length == i) {
       if (media_box)
-	pdf_release_obj(media_box);
+        pdf_release_obj(media_box);
      if (crop_box)
-	pdf_release_obj(crop_box);
+        pdf_release_obj(crop_box);
       goto error;
     }
 
-    if (crop_box)
-      box = crop_box;
-    else
-      if (!(box = pdf_deref_obj(pdf_lookup_dict(page_tree, "ArtBox"))) &&
-	  !(box = pdf_deref_obj(pdf_lookup_dict(page_tree, "TrimBox"))) &&
-	  !(box = pdf_deref_obj(pdf_lookup_dict(page_tree, "BleedBox"))) &&
-	  media_box) {
-	  box = media_box;
-	  media_box = NULL;
+    /* Nasty BBox selection... */
+    if (options == 0) {
+      if (crop_box)
+        box = crop_box;
+      else {
+        if (is_xdv) {
+          /* New scheme only for XDV files */
+          if (!(box = media_box) &&
+              !(box = bleed_box) &&
+              !(box = trim_box) &&
+              art_box) {
+              box = art_box;
+          }
+        } else {
+          /* Backward compatibility */
+          if (!(box = art_box) &&
+              !(box = trim_box) &&
+              !(box = bleed_box) &&
+              media_box) {
+              box = media_box;
+          }
+        }
       }
-    if (media_box)
-      pdf_release_obj(media_box);
+    } else if (options == 1) {
+      if (crop_box)
+        box = crop_box;
+      else
+        if (!(box = media_box) &&
+            !(box = bleed_box) &&
+            !(box = trim_box) &&
+            art_box) {
+            box = art_box;
+        }
+    } else if (options == 2) {
+      if (media_box)
+        box = media_box;
+      else
+        if (!(box = crop_box) &&
+            !(box = bleed_box) &&
+            !(box = trim_box) &&
+            art_box) {
+            box = art_box;
+        }
+    } else if (options == 3) {
+      if (art_box)
+        box = art_box;
+      else
+        if (!(box = crop_box) &&
+            !(box = media_box) &&
+            !(box = bleed_box) &&
+            trim_box) {
+            box = trim_box;
+        }
+    } else if (options == 4) {
+      if (trim_box)
+        box = trim_box;
+      else
+        if (!(box = crop_box) &&
+            !(box = media_box) &&
+            !(box = bleed_box) &&
+            art_box) {
+            box = art_box;
+        }
+    } else if (options == 5) {
+      if (bleed_box)
+        box = bleed_box;
+      else
+        if (!(box = crop_box) &&
+            !(box = media_box) &&
+            !(box = trim_box) &&
+            art_box) {
+            box = art_box;
+        }
+    }
+    medbox = media_box;
   }
 
   if (!PDF_OBJ_ARRAYTYPE(box) || pdf_array_length(box) != 4 ||
@@ -1066,7 +1192,7 @@ pdf_doc_get_page (pdf_file *pf, long page_no, long *count_p,
   if (PDF_OBJ_NUMBERTYPE(rotate)) {
     if (pdf_number_value(rotate))
       WARN("<< /Rotate %d >> found. (Not supported yet)", 
-	   (int) pdf_number_value(rotate));
+           (int) pdf_number_value(rotate));
     pdf_release_obj(rotate);
     rotate = NULL;
   } else if (rotate)
@@ -1079,8 +1205,8 @@ pdf_doc_get_page (pdf_file *pf, long page_no, long *count_p,
       double x;
       pdf_obj *tmp = pdf_deref_obj(pdf_get_array(box, i));
       if (!PDF_OBJ_NUMBERTYPE(tmp)) {
-	pdf_release_obj(tmp);
-	goto error;
+        pdf_release_obj(tmp);
+        goto error;
       }
       x = pdf_number_value(tmp);
       switch (i) {
@@ -1091,6 +1217,26 @@ pdf_doc_get_page (pdf_file *pf, long page_no, long *count_p,
       }
       pdf_release_obj(tmp);
     }
+
+    /* New scheme only for XDV files */
+    if (medbox && (is_xdv || options)) {
+      for (i = 4; i--; ) {
+        double x;
+        pdf_obj *tmp = pdf_deref_obj(pdf_get_array(medbox, i));
+        if (!PDF_OBJ_NUMBERTYPE(tmp)) {
+          pdf_release_obj(tmp);
+          goto error;
+        }
+        x = pdf_number_value(tmp);
+        switch (i) {
+        case 0: if (bbox->llx < x) bbox->llx = x; break;
+        case 1: if (bbox->lly < x) bbox->lly = x; break;
+        case 2: if (bbox->urx > x) bbox->urx = x; break;
+        case 3: if (bbox->ury > x) bbox->ury = x; break;
+        }
+        pdf_release_obj(tmp);
+      }
+    }
   }
 
   pdf_release_obj(box);
@@ -1460,6 +1606,7 @@ pdf_doc_add_goto (pdf_obj *annot_dict)
 {
   pdf_obj *subtype = NULL, *A = NULL, *S = NULL, *D = NULL, *D_new, *dict;
   const char *dest, *key;
+  int destlen = 0;
 
   if (!pdoc.check_gotos)
     return;
@@ -1494,11 +1641,11 @@ pdf_doc_add_goto (pdf_obj *annot_dict)
     else {
       S = pdf_deref_obj(pdf_lookup_dict(A, "S"));
       if (PDF_OBJ_UNDEFINED(S))
-	goto undefined;
+        goto undefined;
       else if (!PDF_OBJ_NAMETYPE(S))
-	goto error;
+        goto error;
       else if (strcmp(pdf_name_value(S), "GoTo"))
-	goto cleanup;
+        goto cleanup;
 
       dict = A;
       key = "D";
@@ -1506,8 +1653,10 @@ pdf_doc_add_goto (pdf_obj *annot_dict)
     }
   }
 
-  if (PDF_OBJ_STRINGTYPE(D))
+  if (PDF_OBJ_STRINGTYPE(D)) {
     dest = (char *) pdf_string_value(D);
+    destlen = pdf_string_length(D);
+  }
 #if 0
   /* Names as destinations are not supported by dvipdfmx */
   else if (PDF_OBJ_NAMETYPE(D))
@@ -1520,16 +1669,16 @@ pdf_doc_add_goto (pdf_obj *annot_dict)
   else
     goto error;
 
-  D_new = ht_lookup_table(&pdoc.gotos, dest, strlen(dest));
+  D_new = ht_lookup_table(&pdoc.gotos, dest, destlen);
   if (!D_new) {
     char buf[10];
 
     /* We use hexadecimal notation for our numeric destinations.
      * Other bases (e.g., 10+26 or 10+2*26) would be more efficient.
      */
-    sprintf(buf, "%lx", ht_table_size(&pdoc.gotos));
+    sprintf(buf, "%x", ht_table_size(&pdoc.gotos));
     D_new = pdf_new_string(buf, strlen(buf));
-    ht_append_table(&pdoc.gotos, dest, strlen(dest), D_new);
+    ht_append_table(&pdoc.gotos, dest, destlen, D_new);
   }
 
   {
@@ -1593,18 +1742,18 @@ pdf_doc_close_names (pdf_doc *p)
     if (p->names[i].data) {
       struct ht_table *data = p->names[i].data;
       pdf_obj  *name_tree;
-      long count;
+      int count;
 
       if (!pdoc.check_gotos || strcmp(p->names[i].category, "Dests"))
-	name_tree = pdf_names_create_tree(data, &count, NULL);
+        name_tree = pdf_names_create_tree(data, &count, NULL);
       else {
-	name_tree = pdf_names_create_tree(data, &count, &pdoc.gotos);
+        name_tree = pdf_names_create_tree(data, &count, &pdoc.gotos);
 
-	if (verbose && count < data->count)
-	  MESG("\nRemoved %ld unused PDF destinations\n", data->count-count);
+        if (verbose && count < data->count)
+          MESG("\nRemoved %ld unused PDF destinations\n", data->count-count);
 
-	if (count < pdoc.gotos.count)
-	  warn_undef_dests(data, &pdoc.gotos);
+        if (count < pdoc.gotos.count)
+          warn_undef_dests(data, &pdoc.gotos);
       }
 
       if (name_tree) {
@@ -1646,10 +1795,11 @@ pdf_doc_close_names (pdf_doc *p)
   return;
 }
 
+static void pdf_doc_get_mediabox (unsigned page_no, pdf_rect *mediabox);
 
 void
 pdf_doc_add_annot (unsigned page_no, const pdf_rect *rect,
-		   pdf_obj *annot_dict, int new_annot)
+                   pdf_obj *annot_dict, int new_annot)
 {
   pdf_doc  *p = &pdoc;
   pdf_page *page;
@@ -1744,19 +1894,11 @@ pdf_doc_begin_article (const char *article_id, pdf_obj *article_info)
   return;
 }
 
-#if 0
-void
-pdf_doc_end_article (const char *article_id)
-{
-  return; /* no-op */
-}
-#endif
-
 static pdf_bead *
 find_bead (pdf_article *article, const char *bead_id)
 {
   pdf_bead *bead;
-  long      i;
+  int       i;
 
   bead = NULL;
   for (i = 0; i < article->num_beads; i++) {
@@ -1771,12 +1913,12 @@ find_bead (pdf_article *article, const char *bead_id)
 
 void
 pdf_doc_add_bead (const char *article_id,
-                  const char *bead_id, long page_no, const pdf_rect *rect)
+                  const char *bead_id, int page_no, const pdf_rect *rect)
 {
   pdf_doc     *p = &pdoc;
   pdf_article *article;
   pdf_bead    *bead;
-  long         i;
+  int          i;
 
   if (!article_id) {
     ERROR("No article identifier specified.");
@@ -1831,7 +1973,7 @@ make_article (pdf_doc *p,
 {
   pdf_obj *art_dict;
   pdf_obj *first, *prev, *last;
-  long     i, n;
+  int      i, n;
 
   if (!article)
     return NULL;
@@ -1924,7 +2066,7 @@ clean_article (pdf_article *article)
     return;
     
   if (article->beads) {
-    long  i;
+    int   i;
 
     for (i = 0; i < article->num_beads; i++) {
       if (article->beads[i].id)
@@ -2004,7 +2146,7 @@ pdf_doc_set_mediabox (unsigned page_no, const pdf_rect *mediabox)
   return;
 }
 
-void
+static void
 pdf_doc_get_mediabox (unsigned page_no, pdf_rect *mediabox)
 {
   pdf_doc  *p = &pdoc;
@@ -2097,16 +2239,16 @@ pdf_doc_get_dictionary (const char *category)
   return dict;
 }
 
-long
+int
 pdf_doc_current_page_number (void)
 {
   pdf_doc *p = &pdoc;
 
-  return (long) (PAGECOUNT(p) + 1);
+  return (int) (PAGECOUNT(p) + 1);
 }
 
 pdf_obj *
-pdf_doc_ref_page (unsigned long page_no)
+pdf_doc_ref_page (unsigned page_no)
 {
   pdf_doc  *p = &pdoc;
   pdf_page *page;
@@ -2124,7 +2266,7 @@ pdf_obj *
 pdf_doc_get_reference (const char *category)
 {
   pdf_obj *ref = NULL;
-  long     page_no;
+  int      page_no;
 
   ASSERT(category);
 
@@ -2376,14 +2518,14 @@ static char *doccreator = NULL; /* Ugh */
 
 void
 pdf_open_document (const char *filename,
-		   int do_encryption,
+                   int enable_encrypt, int enable_objstm,
                    double media_width, double media_height,
                    double annot_grow_amount, int bookmark_open_depth,
                    int check_gotos)
 {
   pdf_doc *p = &pdoc;
 
-  pdf_out_init(filename, do_encryption);
+  pdf_out_init(filename, enable_encrypt, enable_objstm);
 
   pdf_doc_init_catalog(p);
 
@@ -2411,7 +2553,7 @@ pdf_open_document (const char *filename,
 
   pdf_doc_set_bgcolor(NULL);
 
-  if (do_encryption) {
+  if (enable_encrypt) {
     pdf_obj *encrypt = pdf_encrypt_obj();
     pdf_set_encrypt(encrypt);
     pdf_release_obj(encrypt);
@@ -2691,7 +2833,7 @@ pdf_doc_break_annot (void)
     annot_dict = pdf_new_dict();
     pdf_merge_dict(annot_dict, breaking_state.annot_dict);
     pdf_doc_add_annot(pdf_doc_current_page_number(), &(breaking_state.rect),
-		      annot_dict, !breaking_state.broken);
+                      annot_dict, !breaking_state.broken);
     pdf_release_obj(annot_dict);
 
     breaking_state.broken = 1;
@@ -2712,9 +2854,9 @@ pdf_doc_expand_box (const pdf_rect *rect)
 #if 0
 /* This should be number tree */
 void
-pdf_doc_set_pagelabel (long  pg_start,
+pdf_doc_set_pagelabel (int  pg_start,
                        const char *type,
-                       const void *prefix, int prfx_len, long start)
+                       const void *prefix, int prfx_len, int start)
 {
   pdf_doc *p = &pdoc;
   pdf_obj *label_dict;
