@@ -1,6 +1,6 @@
 /* This is dvipdfmx, an eXtended version of dvipdfm by Mark A. Wicks.
 
-    Copyright (C) 2007-2014 by Jin-Hwan Cho and Shunsaku Hirata,
+    Copyright (C) 2007-2016 by Jin-Hwan Cho and Shunsaku Hirata,
     the dvipdfmx project team.
     
     Copyright (C) 1998, 1999 by Mark A. Wicks <mwicks@kettering.edu>
@@ -45,9 +45,10 @@
 
 #include "pdfximage.h"
 
-/* From psimage.h */
-static int  check_for_ps    (FILE *fp);
-static int  ps_include_page (pdf_ximage *ximage, const char *file_name);
+static int  check_for_ps    (FILE *image_file);
+static int  check_for_mp    (FILE *image_file);
+static int  ps_include_page (pdf_ximage *ximage,
+                             const char *ident, load_options options);
 
 
 #define IMAGE_TYPE_UNKNOWN -1
@@ -62,16 +63,22 @@ static int  ps_include_page (pdf_ximage *ximage, const char *file_name);
 
 struct attr_
 {
-  long     width, height;
+  int      width, height;
   double   xdensity, ydensity;
   pdf_rect bbox;
+
+  /* Not appropriate place but... someone need them. */
+  int      page_no;
+  int      page_count;
+  int      bbox_type;  /* Ugh */
+  pdf_obj *dict;
+  char     tempfile;
 };
 
 struct pdf_ximage_
 {
   char        *ident;
   char         res_name[16];
-  long         page_no, page_count;
 
   int          subtype;
 
@@ -80,9 +87,7 @@ struct pdf_ximage_
   char        *filename;
   pdf_obj     *reference;
   pdf_obj     *resource;
-  pdf_obj     *attr_dict;
 
-  char        tempfile;
 };
 
 
@@ -111,34 +116,27 @@ static struct ic_  _ic = {
 };
 
 static void
-pdf_init_ximage_struct (pdf_ximage *I,
-			const char *ident, const char *filename,
-			long page_no, pdf_obj *dict)
+pdf_init_ximage_struct (pdf_ximage *I)
 {
-  if (ident) {
-    I->ident = NEW(strlen(ident)+1, char);
-    strcpy(I->ident, ident);
-  } else
-    I ->ident = NULL;
-  I->page_no  = page_no;
-  I->page_count = 0;
-  if (filename) {
-    I->filename = NEW(strlen(filename)+1, char);
-    strcpy(I->filename, filename);
-  } else
+  I->ident    = NULL;
     I->filename = NULL;
+
   I->subtype  = -1;
   memset(I->res_name, 0, 16);
   I->reference = NULL;
   I->resource  = NULL;
-  I->attr_dict = dict;
 
   I->attr.width = I->attr.height = 0;
   I->attr.xdensity = I->attr.ydensity = 1.0;
   I->attr.bbox.llx = I->attr.bbox.lly = 0;
   I->attr.bbox.urx = I->attr.bbox.ury = 0;
 
-  I->tempfile = 0;
+  I->attr.page_no    = 1;
+  I->attr.page_count = 1;
+  I->attr.bbox_type  = 0;
+
+  I->attr.dict     = NULL;
+  I->attr.tempfile = 0;
 }
 
 static void
@@ -148,7 +146,7 @@ pdf_set_ximage_tempfile (pdf_ximage *I, const char *filename)
     RELEASE(I->filename);
   I->filename = NEW(strlen(filename)+1, char);
   strcpy(I->filename, filename);
-  I->tempfile = 1;
+  I->attr.tempfile = 1;
 }
 
 static void
@@ -162,9 +160,9 @@ pdf_clean_ximage_struct (pdf_ximage *I)
     pdf_release_obj(I->reference);
   if (I->resource)
     pdf_release_obj(I->resource);
-  if (I->attr_dict)
-    pdf_release_obj(I->attr_dict);
-  pdf_init_ximage_struct(I, NULL, NULL, 0, NULL);
+  if (I->attr.dict) /* unsafe? */
+    pdf_release_obj(I->attr.dict);
+  pdf_init_ximage_struct(I);
 }
 
 
@@ -185,19 +183,19 @@ pdf_close_images (void)
     int  i;
     for (i = 0; i < ic->count; i++) {
       pdf_ximage *I = ic->ximages+i;
-      if (I->tempfile) {
-	/*
-	 * It is important to remove temporary files at the end because
-	 * we cache file names. Since we use mkstemp to create them, we
-	 * might get the same file name again if we delete the first file.
-	 * (This happens on NetBSD, reported by Jukka Salmi.)
-	 * We also use this to convert a PS file only once if multiple
-	 * pages are imported from that file.
-	 */
-	if (_opts.verbose > 1 && keep_cache != 1)
-	  MESG("pdf_image>> deleting temporary file \"%s\"\n", I->filename);
-	dpx_delete_temp_file(I->filename, false); /* temporary filename freed here */
-	I->filename = NULL;
+      if (I->attr.tempfile) {
+        /*
+         * It is important to remove temporary files at the end because
+         * we cache file names. Since we use mkstemp to create them, we
+         * might get the same file name again if we delete the first file.
+         * (This happens on NetBSD, reported by Jukka Salmi.)
+         * We also use this to convert a PS file only once if multiple
+         * pages are imported from that file.
+         */
+        if (_opts.verbose > 1 && keep_cache != 1)
+          MESG("pdf_image>> deleting temporary file \"%s\"\n", I->filename);
+        dpx_delete_temp_file(I->filename, false); /* temporary filename freed here */
+        I->filename = NULL;
       }
       pdf_clean_ximage_struct(I);
     }
@@ -254,7 +252,7 @@ source_image_type (FILE *fp)
 
 static int
 load_image (const char *ident, const char *fullname, int format, FILE  *fp,
-            long page_no, pdf_obj *dict)
+            load_options options)
 {
   struct ic_ *ic = &_ic;
   int         id = -1; /* ret */
@@ -267,7 +265,19 @@ load_image (const char *ident, const char *fullname, int format, FILE  *fp,
   }
 
   I  = &ic->ximages[id];
-  pdf_init_ximage_struct(I, ident, fullname, page_no, dict);
+  pdf_init_ximage_struct(I);
+  if (ident) {
+    I->ident = NEW(strlen(ident)+1, char);
+    strcpy(I->ident, ident);
+  }
+  if (fullname) {
+    I->filename = NEW(strlen(fullname)+1, char);
+    strcpy(I->filename, fullname);
+  }
+
+  I->attr.page_no   = options.page_no;
+  I->attr.bbox_type = options.bbox_type;
+  I->attr.dict      = options.dict; /* unsafe? */
 
   switch (format) {
   case  IMAGE_TYPE_JPEG:
@@ -304,25 +314,27 @@ load_image (const char *ident, const char *fullname, int format, FILE  *fp,
     if (_opts.verbose)
       MESG("[PDF]");
     {
-      int result = pdf_include_page(I, fp, fullname);
+      int result = pdf_include_page(I, fp, fullname, options);
       if (result > 0)
-	/* PDF version too recent */
-	result = ps_include_page(I, fullname);
+        /* PDF version too recent */
+        result = ps_include_page(I, fullname, options);
       if (result < 0)
-	goto error;
+        goto error;
     }
     if (_opts.verbose)
-      MESG(",Page:%ld", I->page_no);
+      MESG(",Page:%ld", I->attr.page_no);
     I->subtype  = PDF_XOBJECT_TYPE_FORM;
     break;
-  // case  IMAGE_TYPE_EPS:
+/*
+  case  IMAGE_TYPE_EPS:
+*/
   default:
     if (_opts.verbose)
       MESG(format == IMAGE_TYPE_EPS ? "[PS]" : "[UNKNOWN]");
-    if (ps_include_page(I, fullname) < 0)
+    if (ps_include_page(I, fullname, options) < 0)
       goto error;
     if (_opts.verbose)
-      MESG(",Page:%ld", I->page_no);
+      MESG(",Page:%ld", I->attr.page_no);
     I->subtype  = PDF_XOBJECT_TYPE_FORM;
   }
 
@@ -353,7 +365,7 @@ load_image (const char *ident, const char *fullname, int format, FILE  *fp,
 #define dpx_fclose(f)  (MFCLOSE((f)))
 
 int
-pdf_ximage_findresource (const char *ident, long page_no, pdf_obj *dict)
+pdf_ximage_findresource (const char *ident, load_options options)
 {
   struct ic_ *ic = &_ic;
   int         id = -1;
@@ -362,16 +374,20 @@ pdf_ximage_findresource (const char *ident, long page_no, pdf_obj *dict)
   int         format;
   FILE       *fp;
 
+  /* I don't understand why there is comparision against I->attr.dict here...
+   * I->attr.dict and options.dict are simply pointers to PDF dictionaries.
+   */
   for (id = 0; id < ic->count; id++) {
     I = &ic->ximages[id];
     if (I->ident && !strcmp(ident, I->ident)) {
       f = I->filename;
-      if (I->page_no == page_no + (page_no < 0 ? I->page_count+1 : 0) &&
-          I->attr_dict == dict) {
-	return  id;
+      if (I->attr.page_no == options.page_no /* Not sure */
+          && I->attr.dict == options.dict    /* ????? */
+          && I->attr.bbox_type == options.bbox_type) {
+          return id;
+        }
       }
     }
-  }
 
   if (f) {
     /* we already have converted this file; f is the temporary file name */
@@ -411,7 +427,7 @@ pdf_ximage_findresource (const char *ident, long page_no, pdf_obj *dict)
     } else
       break;
   default:
-    id = load_image(ident, fullname, format, fp, page_no, dict);
+    id = load_image(ident, fullname, format, fp, options);
     break;
   }
   dpx_fclose(fp);
@@ -523,10 +539,10 @@ pdf_ximage_set_image (pdf_ximage *I, void *image_info, pdf_obj *resource)
   pdf_add_dict(dict, pdf_new_name("Width"),   pdf_new_number(info->width));
   pdf_add_dict(dict, pdf_new_name("Height"),  pdf_new_number(info->height));
   if (info->bits_per_component > 0) /* Ignored for JPXDecode filter. FIXME */
-	pdf_add_dict(dict, pdf_new_name("BitsPerComponent"),
-                     pdf_new_number(info->bits_per_component));
-  if (I->attr_dict)
-    pdf_merge_dict(dict, I->attr_dict);
+    pdf_add_dict(dict, pdf_new_name("BitsPerComponent"),
+                 pdf_new_number(info->bits_per_component));
+  if (I->attr.dict)
+    pdf_merge_dict(dict, I->attr.dict);
 
   pdf_release_obj(resource); /* Caller don't know we are using reference. */
   I->resource  = NULL;
@@ -550,10 +566,10 @@ pdf_ximage_set_form (pdf_ximage *I, void *form_info, pdf_obj *resource)
   I->resource  = NULL;
 }
 
-long
+int
 pdf_ximage_get_page (pdf_ximage *I)
 {
-  return I->page_no;
+  return I->attr.page_no;
 }
 
 #define CHECK_ID(c,n) do {\
@@ -581,7 +597,7 @@ pdf_ximage_get_reference (int id)
 /* called from pdfdoc.c only for late binding */
 int
 pdf_ximage_defineresource (const char *ident,
-			   int subtype, void *info, pdf_obj *resource)
+                           int subtype, void *info, pdf_obj *resource)
 {
   struct ic_ *ic = &_ic;
   int         id;
@@ -595,7 +611,12 @@ pdf_ximage_defineresource (const char *ident,
 
   I = &ic->ximages[id];
 
-  pdf_init_ximage_struct(I, ident, NULL, 0, NULL);
+  pdf_init_ximage_struct(I);
+
+  if (ident) {
+    I->ident = NEW(strlen(ident)+1, char);
+    strcpy(I->ident, ident);
+  }
 
   switch (subtype) {
   case PDF_XOBJECT_TYPE_IMAGE:
@@ -642,7 +663,7 @@ pdf_ximage_get_subtype (int id)
 }
 
 void
-pdf_ximage_set_attr (int id, long width, long height, double xdensity, double ydensity, double llx, double lly, double urx, double ury)
+pdf_ximage_set_attr (int id, int width, int height, double xdensity, double ydensity, double llx, double lly, double urx, double ury)
 {
   struct ic_ *ic = &_ic;
   pdf_ximage *I;
@@ -880,7 +901,7 @@ char *get_distiller_template (void)
 }
 
 static int
-ps_include_page (pdf_ximage *ximage, const char *filename)
+ps_include_page (pdf_ximage *ximage, const char *filename, load_options options)
 {
   char  *distiller_template = _opts.cmdtmpl;
   char  *temp;
@@ -941,10 +962,7 @@ ps_include_page (pdf_ximage *ximage, const char *filename)
     return  -1;
   }
   pdf_set_ximage_tempfile(ximage, temp);
-#if 0
-  error = pdf_include_page(ximage, fp, 0, pdfbox_crop);
-#endif
-  error = pdf_include_page(ximage, fp, temp);
+  error = pdf_include_page(ximage, fp, temp, options);
   MFCLOSE(fp);
 
   /* See pdf_close_images for why we cannot delete temporary files here. */
@@ -970,3 +988,25 @@ static int check_for_ps (FILE *image_file)
     return 1;
   return 0;
 }
+
+static int check_for_mp (FILE *image_file) 
+{
+  int try_count = 10;
+
+  rewind (image_file);
+  mfgets(work_buffer, WORK_BUFFER_SIZE, image_file);
+  if (strncmp(work_buffer, "%!PS", 4))
+    return 0;
+
+  while (try_count > 0) {
+    mfgets(work_buffer, WORK_BUFFER_SIZE, image_file);
+    if (!strncmp(work_buffer, "%%Creator:", 10)) {
+      if (strlen(work_buffer+10) >= 8 &&
+	  strstr(work_buffer+10, "MetaPost"))
+	break;
+    }
+    try_count--;
+  }
+
+  return ((try_count > 0) ? 1 : 0);
+}
