@@ -2,7 +2,7 @@
 ** InputReader.cpp                                                      **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2016 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2017 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -22,7 +22,7 @@
 #include <cmath>
 #include <functional>
 #include <vector>
-#include "InputReader.h"
+#include "InputReader.hpp"
 
 using namespace std;
 
@@ -137,7 +137,7 @@ bool InputReader::parseUInt (unsigned &val) {
 }
 
 
-bool InputReader::parseInt (int base, int &val) {
+bool InputReader::parseUInt (int base, unsigned &val) {
 	if (base < 2 || base > 32)
 		return false;
 
@@ -150,7 +150,7 @@ bool InputReader::parseInt (int base, int &val) {
 	val = 0;
 	while (isalnum(c = tolower(peek())) && c <= maxdigit) {
 		get();
-		int digit = c - (c <= '9' ? '0' : 'a'-10);
+		unsigned digit = c - (c <= '9' ? '0' : 'a'-10);
 		val = val*base + digit;
 	}
 	return true;
