@@ -2,7 +2,7 @@
 ** PsSpecialHandler.cpp                                                 **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2016 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2017 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -23,17 +23,17 @@
 #include <fstream>
 #include <memory>
 #include <sstream>
-#include "EPSFile.h"
-#include "FileFinder.h"
-#include "Message.h"
-#include "PathClipper.h"
-#include "PSPattern.h"
-#include "PSPreviewFilter.h"
-#include "PsSpecialHandler.h"
-#include "SpecialActions.h"
-#include "SVGTree.h"
-#include "TensorProductPatch.h"
-#include "TriangularPatch.h"
+#include "EPSFile.hpp"
+#include "FileFinder.hpp"
+#include "Message.hpp"
+#include "PathClipper.hpp"
+#include "PSPattern.hpp"
+#include "PSPreviewFilter.hpp"
+#include "PsSpecialHandler.hpp"
+#include "SpecialActions.hpp"
+#include "SVGTree.hpp"
+#include "TensorProductPatch.hpp"
+#include "TriangularPatch.hpp"
 
 
 using namespace std;
@@ -60,8 +60,8 @@ PsSpecialHandler::PsSpecialHandler () : _psi(this), _actions(0), _previewFilter(
 
 PsSpecialHandler::~PsSpecialHandler () {
 	_psi.setActions(0);     // ensure no further PS actions are performed
-	for (map<int, PSPattern*>::iterator it=_patterns.begin(); it != _patterns.end(); ++it)
-		delete it->second;
+	for (auto &entry : _patterns)
+		delete entry.second;
 }
 
 
@@ -76,7 +76,7 @@ void PsSpecialHandler::initialize () {
 		for (const char **p=headers; *p; ++p)
 			processHeaderFile(*p);
 		// disable bop/eop operators to prevent side-effects by
-		// unexpected bobs/eops present in PS specials
+		// unexpected bops/eops present in PS specials
 		_psi.execute("\nTeXDict begin /bop{pop pop}def /eop{}def end ");
 		_psSection = PS_HEADERS;  // allow to process header specials now
 	}
@@ -101,7 +101,7 @@ void PsSpecialHandler::initgraphics () {
 
 
 void PsSpecialHandler::processHeaderFile (const char *name) {
-	if (const char *path = FileFinder::lookup(name, false)) {
+	if (const char *path = FileFinder::instance().lookup(name, false)) {
 		ifstream ifs(path);
 		_psi.execute(string("%%BeginProcSet: ")+name+" 0 0\n", false);
 		_psi.execute(ifs, false);
@@ -174,12 +174,12 @@ void PsSpecialHandler::executeAndSync (istream &is, bool updatePos) {
 }
 
 
-void PsSpecialHandler::preprocess (const char *prefix, istream &is, SpecialActions *actions) {
+void PsSpecialHandler::preprocess (const char *prefix, istream &is, SpecialActions &actions) {
 	initialize();
 	if (_psSection != PS_HEADERS)
 		return;
 
-	_actions = actions;
+	_actions = &actions;
 	if (*prefix == '!') {
 		_headerCode += "\n";
 		_headerCode += string(istreambuf_iterator<char>(is), istreambuf_iterator<char>());
@@ -193,12 +193,12 @@ void PsSpecialHandler::preprocess (const char *prefix, istream &is, SpecialActio
 }
 
 
-bool PsSpecialHandler::process (const char *prefix, istream &is, SpecialActions *actions) {
+bool PsSpecialHandler::process (const char *prefix, istream &is, SpecialActions &actions) {
 	// process PS headers only once (in prescan)
 	if (*prefix == '!' || strcmp(prefix, "header=") == 0)
 		return true;
 
-	_actions = actions;
+	_actions = &actions;
 	initialize();
 	if (_psSection != PS_BODY)
 		enterBodySection();
@@ -385,73 +385,73 @@ static bool transform_box_extents (const Matrix &matrix, double &w, double &h, d
 }
 
 
-void PsSpecialHandler::dviEndPage (unsigned) {
+void PsSpecialHandler::dviEndPage (unsigned, SpecialActions &actions) {
 	BoundingBox bbox;
 	if (_previewFilter.getBoundingBox(bbox)) {  // is there any data written by preview package?
 		double w = max(0.0, _previewFilter.width());
 		double h = max(0.0, _previewFilter.height());
 		double d = max(0.0, _previewFilter.depth());
-		if (_actions && (_actions->getBBoxFormatString() == "preview" || _actions->getBBoxFormatString() == "min")) {
-			if (_actions->getBBoxFormatString() == "preview") {
-				_actions->bbox() = bbox;
+		if (actions.getBBoxFormatString() == "preview" || actions.getBBoxFormatString() == "min") {
+			if (actions.getBBoxFormatString() == "preview") {
+				actions.bbox() = bbox;
 				Message::mstream() << "\napplying bounding box set by";
 			}
 			else {
 				// compute height, depth and width depending on the
 				// tight bounding box derived from the objects on the page
 				double y0 = bbox.maxY()-h;      // y coordinate of the baseline
-				h = _actions->bbox().maxY()-y0;
+				h = actions.bbox().maxY()-y0;
 				if (h < 0) {
 					h = 0;
-					d = _actions->bbox().height();
+					d = actions.bbox().height();
 				}
 				else {
-					d = y0-_actions->bbox().minY();
+					d = y0-actions.bbox().minY();
 					if (d < 0) {
-						h = _actions->bbox().height();
+						h = actions.bbox().height();
 						d = 0;
 					}
 				}
-				w = _actions->bbox().width();
+				w = actions.bbox().width();
 				Message::mstream() << "\ncomputing extents based on data set by";
 			}
 			Message::mstream() << " preview package (version " << _previewFilter.version() << ")\n";
 
 			// apply page transformations to box extents
 			Matrix pagetrans;
-			_actions->getPageTransform(pagetrans);
+			actions.getPageTransform(pagetrans);
 			bool isBaselineHorizontal = transform_box_extents(pagetrans, w, h, d);
-			_actions->bbox().lock();
+			actions.bbox().lock();
 
-			if (isBaselineHorizontal) {
+			if (!isBaselineHorizontal)
+				Message::mstream() << "can't determine height, width, and depth due to non-horizontal baseline\n";
+			else {
 				const double bp2pt = 72.27/72.0;
 				Message::mstream() <<
 					"width=" << XMLString(w*bp2pt) << "pt, "
 					"height=" << XMLString(h*bp2pt) << "pt, "
 					"depth=" << XMLString(d*bp2pt) << "pt\n";
 			}
-			else
-				Message::mstream() << "can't determine height, width, and depth due to non-horizontal baseline\n";
 		}
 #if 0
 		XMLElementNode *rect = new XMLElementNode("rect");
-		rect->addAttribute("x", _actions->bbox().minX());
-		rect->addAttribute("y", _actions->bbox().minY());
+		rect->addAttribute("x", actions.bbox().minX());
+		rect->addAttribute("y", actions.bbox().minY());
 		rect->addAttribute("width", w);
 		rect->addAttribute("height", h+d);
 		rect->addAttribute("fill", "none");
 		rect->addAttribute("stroke", "red");
 		rect->addAttribute("stroke-width", "0.5");
-		_actions->appendToPage(rect);
+		actions.appendToPage(rect);
 		if (d > 0) {
 			XMLElementNode *line = new XMLElementNode("line");
-			line->addAttribute("x1", _actions->bbox().minX());
-			line->addAttribute("y1", _actions->bbox().minY()+h);
-			line->addAttribute("x2", _actions->bbox().maxX());
-			line->addAttribute("y2", _actions->bbox().minY()+h);
+			line->addAttribute("x1", actions.bbox().minX());
+			line->addAttribute("y1", actions.bbox().minY()+h);
+			line->addAttribute("x2", actions.bbox().maxX());
+			line->addAttribute("y2", actions.bbox().minY()+h);
 			line->addAttribute("stroke", "blue");
 			line->addAttribute("stroke-width", "0.5");
-			_actions->appendToPage(line);
+			actions.appendToPage(line);
 		}
 #endif
 	}
@@ -726,7 +726,7 @@ void PsSpecialHandler::setpattern (vector<double> &p) {
 	else {
 		if (PSUncoloredTilingPattern *pattern = dynamic_cast<PSUncoloredTilingPattern*>(it->second))
 			pattern->setColor(color);
-		it->second->apply(_actions);
+		it->second->apply(*_actions);
 		if (PSTilingPattern *pattern = dynamic_cast<PSTilingPattern*>(it->second))
 			_pattern = pattern;
 		else
@@ -772,7 +772,7 @@ void PsSpecialHandler::clip (Path &path, bool evenodd) {
 	if (path.empty() || !_actions)
 		return;
 
-	Path::WindingRule windingRule = evenodd ? Path::WR_EVEN_ODD : Path::WR_NON_ZERO;
+	Path::WindingRule windingRule = evenodd ? Path::WindingRule::EVEN_ODD : Path::WindingRule::NON_ZERO;
 	path.setWindingRule(windingRule);
 
 	if (!_actions->getMatrix().isIdentity())
@@ -824,11 +824,11 @@ void PsSpecialHandler::shfill (vector<double> &params) {
 
 	// collect common data relevant for all shading types
 	int shadingTypeID = static_cast<int>(params[0]);
-	ColorSpace colorSpace = Color::RGB_SPACE;
+	ColorSpace colorSpace = Color::ColorSpace::RGB;
 	switch (static_cast<int>(params[1])) {
-		case 1: colorSpace = Color::GRAY_SPACE; break;
-		case 3: colorSpace = Color::RGB_SPACE; break;
-		case 4: colorSpace = Color::CMYK_SPACE; break;
+		case 1: colorSpace = Color::ColorSpace::GRAY; break;
+		case 3: colorSpace = Color::ColorSpace::RGB; break;
+		case 4: colorSpace = Color::ColorSpace::CMYK; break;
 	}
 	VectorIterator<double> it = params;
 	it += 2;     // skip shading type and color space
@@ -917,20 +917,20 @@ static void read_patch_data (ShadingPatch &patch, int edgeflag,
 
 class ShadingCallback : public ShadingPatch::Callback {
 	public:
-		ShadingCallback (SpecialActions *actions, XMLElementNode *parent, int clippathID)
+		ShadingCallback (SpecialActions &actions, XMLElementNode *parent, int clippathID)
 			: _actions(actions), _group(new XMLElementNode("g"))
 		{
 			if (parent)
 				parent->append(_group);
 			else
-				actions->appendToPage(_group);
+				actions.appendToPage(_group);
 			if (clippathID > 0)
 				_group->addAttribute("clip-path", XMLString("url(#clip")+XMLString(clippathID)+")");
 		}
 
-		void patchSegment (GraphicsPath<double> &path, const Color &color) {
-			if (!_actions->getMatrix().isIdentity())
-				path.transform(_actions->getMatrix());
+		void patchSegment (GraphicsPath<double> &path, const Color &color) override {
+			if (!_actions.getMatrix().isIdentity())
+				path.transform(_actions.getMatrix());
 
 			// draw a single patch segment
 			ostringstream oss;
@@ -942,7 +942,7 @@ class ShadingCallback : public ShadingPatch::Callback {
 		}
 
 	private:
-		SpecialActions *_actions;
+		SpecialActions &_actions;
 		XMLElementNode *_group;
 };
 
@@ -950,18 +950,18 @@ class ShadingCallback : public ShadingPatch::Callback {
 /** Handle all patch meshes whose patches and their connections can be processed sequentially.
  *  This comprises free-form triangular, Coons, and tensor-product patch meshes. */
 void PsSpecialHandler::processSequentialPatchMesh (int shadingTypeID, ColorSpace colorSpace, VectorIterator<double> &it) {
-	auto_ptr<ShadingPatch> previousPatch;
+	unique_ptr<ShadingPatch> previousPatch;
 	while (it.valid()) {
 		int edgeflag = static_cast<int>(*it++);
 		vector<DPair> points;
 		vector<Color> colors;
-		auto_ptr<ShadingPatch> patch;
+		unique_ptr<ShadingPatch> patch;
 
-		patch = auto_ptr<ShadingPatch>(ShadingPatch::create(shadingTypeID, colorSpace));
+		patch = unique_ptr<ShadingPatch>(ShadingPatch::create(shadingTypeID, colorSpace));
 		read_patch_data(*patch, edgeflag, it, points, colors);
 		patch->setPoints(points, edgeflag, previousPatch.get());
 		patch->setColors(colors, edgeflag, previousPatch.get());
-		ShadingCallback callback(_actions, _xmlnode, _clipStack.topID());
+		ShadingCallback callback(*_actions, _xmlnode, _clipStack.topID());
 #if 0
 		if (bgcolorGiven) {
 			// fill whole patch area with given background color
@@ -978,7 +978,7 @@ void PsSpecialHandler::processSequentialPatchMesh (int shadingTypeID, ColorSpace
 			bbox.transform(_actions->getMatrix());
 			_actions->embed(bbox);
 		}
-		previousPatch = patch;
+		previousPatch = std::move(patch);
 	}
 }
 
@@ -1007,7 +1007,7 @@ void PsSpecialHandler::processLatticeTriangularPatchMesh (ColorSpace colorSpace,
 		vertex.color.set(colorSpace, it);
 	}
 	LatticeTriangularPatch patch(colorSpace);
-	ShadingCallback callback(_actions, _xmlnode, _clipStack.topID());
+	ShadingCallback callback(*_actions, _xmlnode, _clipStack.topID());
 	while (it.valid()) {
 		// read next row
 		for (int i=0; i < verticesPerRow; i++) {
@@ -1237,6 +1237,16 @@ void PsSpecialHandler::ClippingStack::dup (int saveID) {
 
 
 const char** PsSpecialHandler::prefixes () const {
-	static const char *pfx[] = {"header=", "psfile=", "PSfile=", "ps:", "ps::", "!", "\"", "pst:", "PST:", 0};
+	static const char *pfx[] = {
+		"header=",    // read and execute PS header file prior to the following PS statements
+		"psfile=",    // read and execute PS file
+		"PSfile=",    // dito
+		"ps:",        // execute literal PS code wrapped by @beginspecial and @endspecial
+		"ps::",       // execute literal PS code without additional adaption of the drawing position
+		"!",          // execute literal PS header code following this prefix
+		"\"",         // execute literal PS code following this prefix
+		"pst:",       // dito
+		"PST:",       // same as "ps:"
+		0};
 	return pfx;
 }
