@@ -2,8 +2,8 @@
  * Gregorio is a program that translates gabc files to GregorioTeX
  * This header shares definitions between the score parser and lexer.
  *
- * Gregorio score determination in gabc input.
- * Copyright (C) 2006-2015 The Gregorio Project (see CONTRIBUTORS.md)
+ * Gregorio score determination from gabc.
+ * Copyright (C) 2006-2017 The Gregorio Project (see CONTRIBUTORS.md)
  *
  * This file is part of Gregorio.
  * 
@@ -21,8 +21,24 @@
  * this program.  If not, see <http://www.gnu.org/licenses/>. 
  */
 
+#ifndef GABC_SCORE_DETERMINATION_H
+#define GABC_SCORE_DETERMINATION_H
+
+#include "bool.h"
+#include "struct.h"
 #include "gabc.h"
 
+/* The bits in this enum are named to correspond with the _BEGIN/_END tokens */
+typedef enum {
+    SB_I = 0x01,
+    SB_B = 0x02,
+    SB_TT = 0x04,
+    SB_SC = 0x08,
+    SB_UL = 0x10,
+    SB_C = 0x20,
+    SB_ELISION = 0x40
+} gabc_style_bits;
+
 typedef union gabc_score_determination_lval_t {
     char *text;
     char character;
@@ -31,7 +47,18 @@ typedef union gabc_score_determination_lval_t {
 #define YYSTYPE gabc_score_determination_lval_t
 #define YYSTYPE_IS_DECLARED 1
 
-int gabc_score_determination_lex(void);
-#define YY_DECL int gabc_score_determination_lex(void)
+#define YY_DECL \
+    int gabc_score_determination_lex(gabc_style_bits *const styles)
+YY_DECL;
 
 #define YYLTYPE gregorio_scanner_location
+
+void gabc_suppress_extra_custos_at_linebreak(gregorio_score *score);
+void gabc_fix_custos_pitches(gregorio_score *score_to_check);
+bool gabc_check_score_integrity(gregorio_score *score_to_check);
+bool gabc_check_infos_integrity(gregorio_score *score_to_check);
+void gabc_determine_oriscus_orientation(const gregorio_score *score);
+void gabc_determine_punctum_inclinatum_orientation(const gregorio_score *score);
+void gabc_determine_ledger_lines(const gregorio_score *const score);
+
+#endif
