@@ -2,7 +2,7 @@
  * Gregorio is a program that translates gabc files to GregorioTeX
  * This file contains the logic for positioning signs on neumes.
  *
- * Copyright (C) 2008-2015 The Gregorio Project (see CONTRIBUTORS.md)
+ * Copyright (C) 2008-2017 The Gregorio Project (see CONTRIBUTORS.md)
  *
  * This file is part of Gregorio.
  * 
@@ -377,7 +377,7 @@ static gregorio_vposition advise_positioning(const gregorio_glyph *const glyph,
 
     switch (type) {
     case T_PES:
-    case T_PESQUILISMA:
+    case T_QUILISMA_PES:
         /*
          * in the case of a pes, we put the episema just under the bottom note 
          */
@@ -408,9 +408,9 @@ static gregorio_vposition advise_positioning(const gregorio_glyph *const glyph,
             v_episema = below_if_auctus(glyph);
         }
         break;
-    case T_PESQUADRATUM:
-    case T_PESQUASSUS:
-    case T_PESQUILISMAQUADRATUM:
+    case T_PES_QUADRATUM:
+    case T_PES_QUASSUS:
+    case T_QUILISMA_PES_QUADRATUM:
         if (i == 1) {
             note->gtex_offset_case = first_note_case(note, glyph);
             h_episema = above_if_h_episema(note->next);
@@ -909,9 +909,8 @@ static gregorio_vposition advise_positioning(const gregorio_glyph *const glyph,
             note->gtex_offset_case = FinalInclinatumDeminutus;
             break;
         case S_PUNCTUM_INCLINATUM_AUCTUS:
-        case S_PUNCTUM_INCLINATUM:
-        case S_PUNCTUM_CAVUM_INCLINATUM_AUCTUS:
-        case S_PUNCTUM_CAVUM_INCLINATUM:
+        case S_PUNCTUM_INCLINATUM_ASCENDENS:
+        case S_PUNCTUM_INCLINATUM_DESCENDENS:
             note->gtex_offset_case = FinalInclinatum;
             break;
         case S_STROPHA:
@@ -927,11 +926,6 @@ static gregorio_vposition advise_positioning(const gregorio_glyph *const glyph,
             note->gtex_offset_case = fused_single_note_case(glyph, FinalOriscus,
                     LeadingOriscus);
             break;
-        case S_ORISCUS_CAVUM_ASCENDENS:
-        case S_ORISCUS_CAVUM_DESCENDENS:
-        case S_ORISCUS_CAVUM_DEMINUTUS:
-            note->gtex_offset_case = FinalOriscus;
-            break;
         case S_VIRGA:
             note->gtex_offset_case = InitialVirga;
             break;
@@ -943,7 +937,6 @@ static gregorio_vposition advise_positioning(const gregorio_glyph *const glyph,
             }
             break;
         case S_LINEA_PUNCTUM:
-        case S_LINEA_PUNCTUM_CAVUM:
             note->gtex_offset_case = FinalLineaPunctum;
             break;
         case S_LINEA:
@@ -1167,17 +1160,17 @@ static __inline void position_h_episema(gregorio_note *const note,
             adj->pitch_extremum = h->height;
         }
     }
-    if (!note->explicit_high_ledger_line && !note->supposed_high_ledger_line) {
-        note->supposed_high_ledger_line = high_ledger_line;
+    if (!note->high_ledger_specificity && !note->high_ledger_line) {
+        note->high_ledger_line = high_ledger_line;
     }
-    if (!note->explicit_low_ledger_line && !note->supposed_low_ledger_line) {
-        note->supposed_low_ledger_line = low_ledger_line;
+    if (!note->low_ledger_specificity && !note->low_ledger_line) {
+        note->low_ledger_line = low_ledger_line;
     }
 }
 
 static __inline void next_has_ledger_line(
         const height_computation *const h, bool *high_ledger_line,
-        bool *low_ledger_line, const gregorio_score *const score)
+        bool *low_ledger_line)
 {
     const gregorio_element *element = h->last_connected_element;
     const gregorio_glyph *glyph = h->last_connected_glyph;
@@ -1204,10 +1197,8 @@ static __inline void next_has_ledger_line(
                 note = glyph->u.notes.first_note;
             }
 
-            *high_ledger_line = *high_ledger_line
-                    || has_high_ledger_line(note->u.note.pitch, false, score);
-            *low_ledger_line = *low_ledger_line
-                    || has_low_ledger_line(note->u.note.pitch, false);
+            *high_ledger_line = *high_ledger_line || note->high_ledger_line;
+            *low_ledger_line = *low_ledger_line || note->low_ledger_line;
 
             if (keep_going) {
                 keep_going = false;
@@ -1223,7 +1214,7 @@ static __inline void next_has_ledger_line(
 
 static __inline void previous_has_ledger_line(
         const height_computation *const h, bool *high_ledger_line,
-        bool *low_ledger_line, const gregorio_score *const score)
+        bool *low_ledger_line)
 {
     const gregorio_element *element = h->start_element;
     const gregorio_glyph *glyph = h->start_glyph;
@@ -1252,10 +1243,8 @@ static __inline void previous_has_ledger_line(
                 } while (glyph->type != GRE_GLYPH);
                 note = gregorio_glyph_last_note(glyph);
             }
-            *high_ledger_line = *high_ledger_line
-                    || has_high_ledger_line(note->u.note.pitch, false, score);
-            *low_ledger_line = *low_ledger_line
-                    || has_low_ledger_line(note->u.note.pitch, false);
+            *high_ledger_line = *high_ledger_line || note->high_ledger_line;
+            *low_ledger_line = *low_ledger_line || note->low_ledger_line;
 
             if (keep_going) {
                 keep_going = false;
@@ -1283,8 +1272,8 @@ static __inline void set_h_episema_height(const height_computation *const h,
     bool low_ledger_line = has_low_ledger_line(h->height, true)
             || has_low_ledger_line(h->height - h->vpos, false);
 
-    next_has_ledger_line(h, &high_ledger_line, &low_ledger_line, score);
-    previous_has_ledger_line(h, &high_ledger_line, &low_ledger_line, score);
+    next_has_ledger_line(h, &high_ledger_line, &low_ledger_line);
+    previous_has_ledger_line(h, &high_ledger_line, &low_ledger_line);
 
     for ( ; element; element = element->next) {
         if (element->type == GRE_ELEMENT) {
@@ -1327,19 +1316,26 @@ static __inline bool is_connected_right(const grehepisema_size size) {
     return size == H_NORMAL || size == H_SMALL_RIGHT;
 }
 
-static __inline bool is_connectable_interglyph_ambitus(
+static __inline bool is_connectable_interglyph_ambitus(const signed char first,
+        const signed char second)
+{
+    return abs(first - second) < 3;
+}
+
+static __inline bool is_connectable_interglyph_notes(
         const gregorio_note *const first, const gregorio_note *const second)
 {
-    return first && second
-            && abs(first->u.note.pitch - second->u.note.pitch) < 3;
+    return first && second && is_connectable_interglyph_ambitus(
+            first->u.note.pitch, second->u.note.pitch);
 }
 
 static __inline bool has_space_to_left(const gregorio_note *const note) {
     switch (note->u.note.shape) {
-    case S_PUNCTUM_INCLINATUM:
+    case S_PUNCTUM_INCLINATUM_ASCENDENS:
+    case S_PUNCTUM_INCLINATUM_DESCENDENS:
     case S_PUNCTUM_INCLINATUM_DEMINUTUS:
     case S_PUNCTUM_INCLINATUM_AUCTUS:
-        return !is_connectable_interglyph_ambitus(note->previous, note);
+        return !is_connectable_interglyph_notes(note->previous, note);
 
     default:
         return !note->previous;
@@ -1428,7 +1424,6 @@ static __inline void compute_h_episema(height_computation *const h,
         const gregorio_glyph *const glyph, gregorio_note *const note,
         const int i, const gtex_type type, const gregorio_score *const score)
 {
-    signed char next_height;
     grehepisema_size size;
 
     if (h->is_applicable(note)) {
@@ -1436,11 +1431,11 @@ static __inline void compute_h_episema(height_computation *const h,
             size = h->get_size(note);
 
             if (h->active) {
+                const signed char next_height = compute_h_episema_height(
+                        glyph, note, h->vpos);
                 if (h->connected && is_connected_left(size)
                         && (i != 1 || is_connectable_interglyph_ambitus(
-                                note, h->last_connected_note))) {
-                    next_height = compute_h_episema_height(glyph, note,
-                            h->vpos);
+                                next_height, h->height))) {
                     if (h->is_better_height(next_height, h->height)) {
                         h->height = next_height;
                     }
@@ -1553,10 +1548,6 @@ static __inline int compute_fused_shift(const gregorio_glyph *glyph)
     switch (first_note->u.note.shape) {
     case S_QUILISMA:
     case S_QUILISMA_QUADRATUM:
-    case S_PUNCTUM_CAVUM:
-    case S_ORISCUS_CAVUM_ASCENDENS:
-    case S_ORISCUS_CAVUM_DESCENDENS:
-    case S_ORISCUS_CAVUM_DEMINUTUS:
     case S_FLAT:
     case S_SHARP:
     case S_NATURAL:
@@ -1585,22 +1576,6 @@ static __inline int compute_fused_shift(const gregorio_glyph *glyph)
     gregorio_assert(prev_note->type == GRE_NOTE, compute_fused_shift,
             "previous note wasn't a note", return 0);
 
-    switch (prev_note->u.note.shape) {
-    case S_PUNCTUM_CAVUM:
-    case S_ORISCUS_CAVUM_ASCENDENS:
-    case S_ORISCUS_CAVUM_DESCENDENS:
-    case S_ORISCUS_CAVUM_DEMINUTUS:
-    case S_FLAT:
-    case S_SHARP:
-    case S_NATURAL:
-        /* these don't fuse to anything */
-        return 0;
-
-    default:
-        /* anything else is potentially fusible */
-        break;
-    }
-
     shift = first_note->u.note.pitch - prev_note->u.note.pitch;
     gregorio_assert(shift >= -MAX_AMBITUS && shift <= MAX_AMBITUS,
             compute_fused_shift, "ambitus too large to fuse", return 0);
@@ -1610,120 +1585,38 @@ static __inline int compute_fused_shift(const gregorio_glyph *glyph)
         return 0;
     }
 
-    /* the FLEXA check below checks for a porrectus-like flexus, which is not
-     * fusible from above */
-    if (shift < 0 && ((next_is_fused && glyph->u.notes.glyph_type == G_FLEXA)
-                || glyph->u.notes.glyph_type == G_PORRECTUS
-                || (glyph->u.notes.glyph_type == G_PODATUS
-                    && !(glyph->u.notes.liquescentia & L_DEMINUTUS))
-                || (previous->u.notes.glyph_type == G_PUNCTUM
-                    && is_initio_debilis(previous->u.notes.liquescentia)))) {
-        /* may not be fused from above */
-        return 0;
-    }
-
-    /* Special cases for oriscus */
-    switch (first_note->u.note.shape) {
-        gregorio_note *next_note;
-        const gregorio_glyph *next_glyph;
-    case S_ORISCUS_ASCENDENS:
-    case S_ORISCUS_DESCENDENS:
-    case S_ORISCUS_SCAPUS_ASCENDENS:
-    case S_ORISCUS_SCAPUS_DESCENDENS:
-        next_note = first_note->next;
-        if (!next_note && (next_glyph = gregorio_next_non_texverb_glyph(glyph))
-                && next_glyph->type == GRE_GLYPH
-                && is_fused(next_glyph->u.notes.liquescentia)) {
-            next_note = next_glyph->u.notes.first_note;
-        }
-        if (next_note) {
-            if (next_note->u.note.pitch < first_note->u.note.pitch) {
-                /* then this note should be an down-up-down oriscus */
-                if (shift > 0) {
-                    /* down-up-down oricus cannot be fused from below */
-                    return 0;
-                }
-            } else {
-                /* then this note should be an up-down-up oriscus */
-                if (shift < 0) {
-                    /* up-down-up oricus cannot be fused from above */
-                    return 0;
-                }
+    if (shift < 0) {
+        if (glyph->u.notes.glyph_type == G_PODATUS) {
+            if (glyph->u.notes.liquescentia & L_DEMINUTUS) {
+                /* podatus deminutus may be fused from above */
+                return shift;
+            }
+            switch (first_note->u.note.shape) {
+            case S_ORISCUS_ASCENDENS:
+            case S_ORISCUS_DESCENDENS:
+            case S_ORISCUS_SCAPUS_ASCENDENS:
+            case S_ORISCUS_SCAPUS_DESCENDENS:
+                return shift;
+            default:
+                /* otherwise, podatus may not be fused from above */
+                return 0;
             }
         }
-        break;
 
-    default:
-        break;
+        /* the FLEXA check below checks for a porrectus-like flexus, which is
+         * not fusible from above */
+        if ((next_is_fused && glyph->u.notes.glyph_type == G_FLEXA)
+                || glyph->u.notes.glyph_type == G_PORRECTUS
+                || (previous->u.notes.glyph_type == G_PUNCTUM
+                    && is_initio_debilis(previous->u.notes.liquescentia))) {
+            /* may not be fused from above */
+            return 0;
+        }
     }
 
     return shift;
 }
 
-static __inline void guess_ledger_lines(const gregorio_element *element,
-        const gregorio_score *const score)
-{
-    bool high_ledger_line = false;
-    bool low_ledger_line = false;
-    gregorio_note *prev = NULL;
-
-    for (; element; element = element->next) {
-        if (element->type == GRE_ELEMENT) {
-            gregorio_glyph *glyph;
-            for (glyph = element->u.first_glyph; glyph;
-                    glyph = glyph->next) {
-                if (glyph->type == GRE_GLYPH) {
-                    gregorio_note *note;
-                    for (note = glyph->u.notes.first_note; note;
-                            note = note->next) {
-                        if (note->type == GRE_NOTE) {
-                            if (high_ledger_line
-                                    && !note->explicit_high_ledger_line
-                                    && !note->supposed_high_ledger_line) {
-                                note->supposed_high_ledger_line = true;
-                            }
-                            if (low_ledger_line
-                                    && !note->explicit_low_ledger_line
-                                    && !note->supposed_low_ledger_line) {
-                                note->supposed_low_ledger_line = true;
-                            }
-                            high_ledger_line = has_high_ledger_line(
-                                    note->u.note.pitch, false, score);
-                            low_ledger_line = has_low_ledger_line(
-                                    note->u.note.pitch, false);
-                            if (high_ledger_line) {
-                                if (!note->explicit_high_ledger_line
-                                        && !note->supposed_high_ledger_line) {
-                                    note->supposed_high_ledger_line = true;
-                                }
-                                if (prev && !prev->explicit_high_ledger_line
-                                        && !prev->supposed_high_ledger_line) {
-                                    prev->supposed_high_ledger_line = true;
-                                }
-                            }
-                            if (low_ledger_line) {
-                                if (!note->explicit_low_ledger_line
-                                        && !note->supposed_low_ledger_line) {
-                                    note->supposed_low_ledger_line = true;
-                                }
-                                if (prev && !prev->explicit_low_ledger_line
-                                        && !prev->supposed_low_ledger_line) {
-                                    prev->supposed_low_ledger_line = true;
-                                }
-                            }
-                            prev = note;
-                        }
-                    }
-                }
-            }
-            /* this heuristic ends eith the element */
-            high_ledger_line = false;
-            low_ledger_line = false;
-            prev = NULL;
-        }
-    }
-}
-
 void gregoriotex_compute_positioning(
         const gregorio_element *const param_element,
         const gregorio_score *const score)
@@ -1775,8 +1668,6 @@ void gregoriotex_compute_positioning(
     gtex_type type;
     const gregorio_element *element;
 
-    guess_ledger_lines(param_element, score);
-
     for (element = param_element; element; element = element->next) {
         if (element->type == GRE_ELEMENT) {
             gregorio_glyph *glyph;
