@@ -1,11 +1,11 @@
 # TeXLive::TLConfig.pm - module exporting configuration values
-# Copyright 2007-2016 Norbert Preining
+# Copyright 2007-2017 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 
 package TeXLive::TLConfig;
 
-my $svnrev = '$Revision: 40652 $';
+my $svnrev = '$Revision: 44018 $';
 my $_modulerevision = ($svnrev =~ m/: ([0-9]+) /) ? $1 : "unknown";
 sub module_revision { return $_modulerevision; }
 
@@ -51,16 +51,13 @@ BEGIN {
 
 # the year of our release, will be used in the location of the
 # network packages, and in menu names, and other places.
-$ReleaseYear = 2016;
+$ReleaseYear = 2017;
 
 # users can upgrade from this year to the current year; might be the
 # same as the release year, or any number of releases earlier.
 # Generally not tested.
 $MinRelease = 2016;
 
-# users can NOT upgrade due to internal changes, force a full installation
-#$MinRelease = $ReleaseYear;
-
 # Meta Categories do not ship files, but only call for other packages.
 our @MetaCategories = qw/Collection Scheme/;
 our $MetaCategoriesRegexp = '(Collection|Scheme)';
@@ -151,6 +148,9 @@ our %TLPDBOptions = (
   "file_assocs" =>
     [ "n:0..2", 1, "fileassocs",
       "Change file associations (w32)" ],
+  "generate_updmap" =>
+    [ "b", 0, "generate_updmap",
+      "Run tlmgr generate updmap after maps have changed" ],
   "install_docfiles" =>
     [ "b", 1, "docfiles",
       "Install documentation files" ],
@@ -175,9 +175,6 @@ our %TLPDBOptions = (
   "w32_multi_user" =>
     [ "b", 1, "multiuser",
       "Install for all users (w32)" ],
-  "generate_updmap" =>
-    [ "b", 0, "generate_updmap",
-      "Run tlmgr generate updmap after maps have changed" ],
 );
 
 
@@ -211,14 +208,14 @@ our $F_NOPOSTACTION = 4;
 # The length of a checksum generated by tlchecksum and contained in
 # texlive.tlpdb.checksum. Needs to be in agreement with the hash
 # method used in TLCrypto::tlchecksum and TLCrypto::tldigest
-$ChecksumLength = 128;
+our $ChecksumLength = 128;
 
 # The program generating the checksum for the file given as first
 # argument
-$ChecksumProgram = "sha512sum";
+our $ChecksumProgram = "sha512sum";
 
 # The extension of the checksum file
-$ChecksumExtension = "sha512";
+our $ChecksumExtension = "sha512";
 
 1;
 
