@@ -8,6 +8,10 @@
  *     Werner Lemberg <wl@gnu.org>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -148,11 +152,11 @@ upmap(Font *fnt)
   for (Ti = fnt->charlist; Ti; Ti = Ti->next)
   {
     p = Ti->adobename;
-    if (isupper(*p))
+    if (isupper((unsigned char)*p))
     {
       q = lwr;
       for (; *p; p++)
-        *q++ = tolower(*p);
+        *q++ = tolower((unsigned char)*p);
       *q = '\0';
 
       if (NULL != (ti = findmappedadobe(lwr, fnt->inencptrs)))
