@@ -63,7 +63,7 @@ typedef struct saved_tex_scanner {
 
 static lua_token *check_istoken(lua_State * L, int ud);
 
-#define TOKEN_METATABLE  "luatex_token"
+#define TOKEN_METATABLE  "luatex.token"
 
 #define DEBUG 0
 #define DEBUG_OUT stdout
@@ -534,7 +534,7 @@ static int run_build(lua_State * L)
     if (lua_type(L, 1) == LUA_TNUMBER) {
         int cs = 0;
         int chr = (int) lua_tointeger(L, 1);
-        int cmd = (int) luaL_optinteger(L, 2, get_cat_code(int_par(cat_code_table_code),chr));
+        int cmd = (int) luaL_optinteger(L, 2, get_cat_code(cat_code_table_par,chr));
         if (cmd == 0 || cmd == 9 || cmd == 14 || cmd == 15) {
             formatted_warning("token lib","not a good token, catcode %i can not be returned, so 12 will be used",(int) cmd);
             cmd = 12;
@@ -797,6 +797,53 @@ static int run_scan_token(lua_State * L)
 
 /* TODO: check for a quick way to set a macro to empty (HH) */
 
+static int get_meaning(lua_State * L)
+{
+    const char *name = null;
+    size_t lname = 0;
+    int cs, cmd;
+    if (lua_type(L, 1) == LUA_TSTRING) {
+        name = lua_tolstring(L, 1, &lname);
+        cs = string_lookup(name, lname);
+        cmd = eq_type(cs);
+        if (cmd >= call_cmd) {
+            int chr = equiv(cs);
+            char *str = tokenlist_to_cstring(chr, true, NULL);
+            lua_pushstring(L, str);
+            free(str);
+            return 1;
+        }
+    }
+    return 0;
+}
+
+static int get_macro(lua_State * L)
+{
+    const char *name = null;
+    size_t lname = 0;
+    int cs, cmd;
+    if (lua_type(L, 1) == LUA_TSTRING) {
+        name = lua_tolstring(L, 1, &lname);
+        cs = string_lookup(name, lname);
+        cmd = eq_type(cs);
+        if (cmd >= call_cmd) {
+            /*
+                Expanding would expand in-place, unless we make a copy which we don't want to
+                do. So, we just pass the meaning i.e. no: expand_macros_in_tokenlist(chr).
+
+                Actually it would be nice to adapt tokenlist_to_cstring with an extra argument
+                indicating that we are not interested in the before -> part.
+            */
+            int chr = equiv(cs);
+            char *str = tokenlist_to_xstring(chr, true, NULL);
+            lua_pushstring(L, str);
+            free(str);
+            return 1;
+        }
+    }
+    return 0;
+}
+
 static int set_macro(lua_State * L)
 {
     const char *name = null;
@@ -821,7 +868,7 @@ static int set_macro(lua_State * L)
         if (n > 3)
             s = lua_tostring(L, 4);
     } else {
-        ct = int_par(cat_code_table_code) ;
+        ct = cat_code_table_par;
         name = lua_tolstring(L, 1, &lname);
         if (n > 1)
             str = lua_tolstring(L, 2, &lstr);
@@ -950,6 +997,8 @@ static const struct luaL_Reg tokenlib[] = {
     { "get_protected", lua_tokenlib_get_protected },
     /* maybe more setters */
     { "set_macro", set_macro },
+    { "get_macro", get_macro },
+    { "get_meaning", get_meaning },
     /* probably never */
  /* {"expand", run_expand},               */ /* does not work yet! */
  /* {"csname_id", run_get_csname_id},     */ /* yes or no */
