@@ -27,6 +27,7 @@ along with Omega; if not, write to the Free Software Foundation, Inc.,
 #include "ligkern_routines.h"
 #include "char_routines.h"
 #include "out_routines.h"
+#include "parse_ofm.h"
 #include "print_routines.h"
 #include "error_routines.h"
 #include "header_routines.h"
@@ -41,11 +42,15 @@ unsigned no_kerns = 0;
 
 unsigned min_nl=0;
 unsigned bchar = CHAR_BOUNDARY;
-unsigned bchar_label = 0;
+#define MAX_LABEL 0x7fffffff
+unsigned bchar_label = MAX_LABEL;
 unsigned bchar_remainder;
 unsigned lk_step_ended=FALSE;
 
-four_entries lig_kern_table[100000];
+#define LIG_KERN_CHUNK 512
+four_entries *lig_kern_table;
+unsigned char *activity;
+static int lig_kern_size;
 
 void
 set_boundary_character(unsigned c)
@@ -60,6 +65,20 @@ init_ligkern(void)
     lk_step_ended = FALSE;
     nl = 0;
     min_nl = 0;
+    lig_kern_size = LIG_KERN_CHUNK;
+    lig_kern_table = (four_entries *) xcalloc(lig_kern_size, sizeof(four_entries));
+}
+
+static void
+lig_kern_incr(void)
+{
+    nl++;
+    if (nl < lig_kern_size)
+        return;
+    lig_kern_size += LIG_KERN_CHUNK;
+    lig_kern_table = (four_entries *)
+                     xrealloc(lig_kern_table, lig_kern_size * sizeof(four_entries));
+    memset(lig_kern_table + nl, 0, LIG_KERN_CHUNK * sizeof(four_entries));
 }
 
 void
@@ -70,19 +89,19 @@ set_label_command(unsigned c)
     } else {
         check_char_tag(c);
 	set_char_tag(c, TAG_LIG);
-        if (nl==0) set_char_remainder(c, 0);
-	else set_char_remainder(c, nl);
+	set_char_remainder(c, nl);
     }
     if (min_nl <= nl) { min_nl = nl+1; }
     lk_step_ended = FALSE;
     no_labels++;
+    
 }
 
 void
 set_stop_command(void)
 {
     if (lk_step_ended == TRUE) {
-        lig_kern_table[nl-1].entries[0] = 
+        lig_kern_table[nl-1].entries[0] =
         lig_kern_table[nl-1].entries[0] / 256 * 256 + STOP_FLAG;
     } else {
         warning_0("STOP must follow LIG or KRN; ignored");
@@ -94,7 +113,7 @@ void
 set_skip_command(unsigned val)
 {
     if (lk_step_ended == TRUE) {
-        if (val>128) {
+        if (val>127) {
             warning_0("Maximum SKIP amount is 127; ignored");
         } else {
             lig_kern_table[nl-1].entries[0] = val;
@@ -113,11 +132,13 @@ set_ligature_command(unsigned lig, unsigned c, unsigned val)
     lig_kern_table[nl].entries[1] = c;
     lig_kern_table[nl].entries[2] = lig;
     lig_kern_table[nl].entries[3] = val;
-    nl++;
+    lig_kern_incr();
     lk_step_ended = TRUE;
 
 }
 
+static int  set_new_kern(fix);
+
 void
 set_kerning_command(unsigned c, fix fval)
 {
@@ -132,7 +153,7 @@ set_kerning_command(unsigned c, fix fval)
         lig_kern_table[nl].entries[2] = KERN_FLAG + (k/65536);
         lig_kern_table[nl].entries[3] = k % 65536;
     }
-    nl++;
+    lig_kern_incr();
     lk_step_ended = TRUE;
 }
 
@@ -166,7 +187,7 @@ set_c_penglue_command(unsigned new_class, unsigned pen_index, unsigned glue_inde
     fatal_error_0("CPENGLUE not currently supported");
 }
 
-int
+static int
 set_new_kern(fix fval)
 {
     unsigned index;
@@ -186,6 +207,8 @@ set_new_kern(fix fval)
             index = lattr(L1);
         } else {
             index = nk++;
+            if (index == (ofm_level==OFM_TFM ? 0x8000 : 0x800000))
+                fatal_error_1("more than %d different kerns", index);
             L2 = av_list1(index, fval);
             L1->ptr = L2;
         }
@@ -209,23 +232,6 @@ build_kern_table(void)
     }
 }
 
-void
-print_kerns(void)
-{
-    unsigned i;
-
-    if (nk>0) {
-        left(); out("COMMENT"); out_ln();
-        for (i=0; i<nk; i++) {
-            left(); out("KERN_ENTRY");
-            out(" "); out_int(i,10);
-            out(" "); out_fix(kern_table[i]);
-            right();
-        }
-        right();
-    }
-}
-
 void
 print_ligkern_table(void)
 {
@@ -235,12 +241,39 @@ print_ligkern_table(void)
     if (nl>0) {
         left(); out("LIGTABLE"); out_ln();
         for (i=0; i<nl; i++) {
-            while ((sort_ptr<=label_ptr) && (i==label_table[sort_ptr].rr)) {
-                print_label_command(label_table[sort_ptr].cc);
-                sort_ptr++;
+            unsigned r = lig_kern_table[i].entries[0];
+            if (r >= 256)
+                activity[i] = A_ACCESSIBLE;
+            else if ((activity[i] == A_ACCESSIBLE) && (r < STOP_FLAG)) {
+                r += i + 1;
+                if (r >= nl) {
+                    fprintf(stderr, "Bad OFM file: Ligature/kern step %u skips too far;\n"
+                                    "I made it stop.\n", i);
+                    lig_kern_table[i].entries[0] = STOP_FLAG;
+                    changed = TRUE;
+                } else
+                    activity[r] = A_ACCESSIBLE;
             }
-            print_one_lig_kern_entry(lig_kern_table+i, TRUE);
         }
+        for (i=0; i<nl; i++) {
+            if (activity[i] == A_UNREACHABLE) {
+                if (parenthesis_level == 1) {
+                    left();
+                    out("COMMENT THIS PART OF THE PROGRAM IS NEVER USED!");
+                    out_ln();
+                }
+            } else if (parenthesis_level == 2)
+                right();
+            if (activity[i] != A_PASS_THROUGH) {
+                while ((sort_ptr<=label_ptr) && (i==label_table[sort_ptr].rr)) {
+                    print_label_command(label_table[sort_ptr].cc);
+                    sort_ptr++;
+                }
+                print_one_lig_kern_entry(lig_kern_table+i, TRUE);
+            }
+        }
+        if (parenthesis_level == 2)	/* the final step was unreachable */
+            right();
         right();
     }
 }
@@ -249,23 +282,31 @@ void
 print_one_lig_kern_entry(four_entries *lentry, boolean show_stop)
 {
     if (lentry->entries[2] >= KERN_FLAG) {
-        if (ofm_level==OFM_TFM) {
-        print_kerning_command(lentry->entries[1],
-        kern_table[256*(lentry->entries[2]-KERN_FLAG)+lentry->entries[3]]);
-        } else {
-        print_kerning_command(lentry->entries[1],
-        kern_table[65536*(lentry->entries[2]-KERN_FLAG)+lentry->entries[3]]);
+        unsigned r = (ofm_level==OFM_TFM ? 256 : 65536)*(lentry->entries[2]-KERN_FLAG)+lentry->entries[3];
+        fix v;
+        if (r < nk)
+            v = kern_table[r];
+        else {
+            if (show_stop == TRUE)
+                fprintf(stderr, "Bad OFM file: Kern index too large.\n");
+            v = 0;
+            changed = TRUE;
         }
+        print_kerning_command(lentry->entries[1], v);
     } else {
         print_ligature_command(lentry->entries[2],
                                lentry->entries[1],
                                lentry->entries[3]);
     }
-    if ((show_stop == TRUE) && (lentry->entries[0] > 0)) {
+    if ((show_stop == TRUE) && (lentry->entries[0] > 0) && (parenthesis_level == 1)) {
         if (lentry->entries[0] >= STOP_FLAG) {
             print_stop_command();
-        } else {
-            print_skip_command(0);
+        } else {	/* count number of accessible steps */
+            unsigned i, k = lentry-lig_kern_table, count = 0;
+            for (i=1; i<=lentry->entries[0]; i++)
+                if (activity[k+i] == A_ACCESSIBLE)
+                    count++;
+            print_skip_command(count);
         }
     }
 }
@@ -275,7 +316,9 @@ hash_list hash_table[PRIME];
 unsigned x_lig_cycle;
 unsigned y_lig_cycle = CHAR_BOUNDARY;
 
-int
+static int l_eval(unsigned, unsigned);
+
+static int
 l_f(hash_list h, unsigned x, unsigned y)
 {
     switch(h->new_class) {
@@ -312,20 +355,20 @@ l_f(hash_list h, unsigned x, unsigned y)
     return (h->lig_z);
 }
 
-int
+static hash_list l_hash_lookup(unsigned, unsigned);
+
+static int
 l_eval(unsigned x, unsigned y)
 {
     hash_list h;
-    
+
     if ((x==CHAR_ERROR) || (y==CHAR_ERROR)) return CHAR_ERROR;
     h = l_hash_lookup(x, y);
     if (h == NULL) return y;
     return l_f(h, x, y);
 }
 
-queue hash_entries;
-
-int
+static int
 l_hash_input(unsigned p, unsigned c)
 {
 
@@ -335,7 +378,7 @@ l_hash_input(unsigned p, unsigned c)
     unsigned cc = LIG_SIMPLE;
     unsigned zz = entry->entries[3];
     unsigned key;
-    hash_list L1, L2;
+    hash_list L1;
 
     if (t >= KERN_FLAG) zz = y;
     else {
@@ -354,49 +397,33 @@ l_hash_input(unsigned p, unsigned c)
         }
     }
     key = (c & 0x7fff)*(y & 0x7fff) % PRIME;
-    if (hash_table[key] == NULL) {
-        hash_table[key] = hash_list1(c,y,cc,zz);
-        append_to_queue(&hash_entries, hash_table[key]);
-    } else {
-        L1 = hash_table[key];
-        L2 = L1->ptr;
-        while ((L2 != NULL) &&
-               ((L2->x <= c) || ((L2->x == c) && (L2->y <= y)))) {
+    L1 = hash_table[key];
+    if ((L1 == NULL) || (c < L1->x) || ((c == L1->x) && (y < L1->y)))
+        hash_table[key] = hash_list1(c,y,cc,zz,L1);
+    else {
+        hash_list L2 = L1->ptr;
+        while ((L2 != NULL) && ((c > L2->x) || ((c == L2->x) && (y > L2->y)))) {
             L1 = L2;
             L2 = L2->ptr;
         }
-        if (L2 == NULL) {
-            L2 = hash_list1(c,y,cc,zz);
-            append_to_queue(&hash_entries, L2);
-            L2->ptr = L1->ptr;
-            L1->ptr = L2;
-        } else if ((c < L2->x) || ((c == L2->x) && (y < L2->y))) {
-            hash_table[key] = hash_list1(c,y,cc,zz);
-            hash_table[key]->ptr = L1;
-            append_to_queue(&hash_entries, hash_table[key]);
-        } else if ((c == L2->x) && (y < L2->y)) {
-            return FALSE; /* unused ligature command */
-        } else {
-            L2 = hash_list1(c,y,cc,zz);
-            append_to_queue(&hash_entries, L2);
-            L2->ptr = L1->ptr;
-            L1->ptr = L2;
-        }
+        if ((L2 == NULL) || (c < L2->x) || ((c == L2->x) && (y < L2->y)))
+            L1->ptr = hash_list1(c,y,cc,zz,L2);
+        else
+            return FALSE;
     }
     return TRUE;
 }
 
-hash_list
+static hash_list
 l_hash_lookup(unsigned x, unsigned y)
 {
     unsigned key = (x & 0x7fff)*(y & 0x7fff) % PRIME;
     hash_list L = hash_table[key];
 
-    if (L==NULL) return NULL;
-    while (L->x < x) L = L->ptr;
-    if (L->x != x) return NULL;
-    while ((L->x == x) && (L->y < y)) L = L->ptr;
-    if (L->y != y) return NULL;
+    while ((L != NULL) && ((x > L->x) || ((x == L->x) && (y > L->y))))
+        L = L->ptr;
+    if ((L == NULL) || (x < L->x) || ((x == L->x) && (y < L->y)))
+        return NULL;
     return L;
 }
 
@@ -404,15 +431,17 @@ void
 check_ligature_ends_properly(void)
 {
     if (nl>0) {
-        if (bchar_label != 0) {
+        if (bchar_label < nl) {
             /* make room for it; the actual label will be stored later */
-            nl++;
+            lig_kern_table[nl].entries[0] = 255;
+            lig_kern_incr();
         }
         while (min_nl > nl) {
-            nl++;
+            lig_kern_table[nl].entries[0] = 255;
+            lig_kern_incr();
         }
-        if (lig_kern_table[nl].entries[0] == 0) {
-            lig_kern_table[nl].entries[0] = STOP_FLAG;
+        if (lig_kern_table[nl-1].entries[0] == 0) {
+            lig_kern_table[nl-1].entries[0] = STOP_FLAG;
         }
     }
 }
@@ -427,22 +456,38 @@ check_ligature_program(unsigned c, unsigned lab)
         entry = lig_kern_table+lig_ptr;
         if (l_hash_input(lig_ptr,c)) {
             if (entry->entries[2] < KERN_FLAG) {
-                if (entry->entries[1] != bchar) 
+                if (entry->entries[1] != bchar) {
                     check_existence_and_safety(c, entry->entries[1],
                         ligature_commands[entry->entries[2]],
                         "%s character examined by (H %X)");
+                    if (entry->entries[1] > 0xffff) {
+                        fprintf(stderr, "%s character (H %X) examined by (H %X) "
+                            "exceeds ffff\n", ligature_commands[entry->entries[2]], entry->entries[1], c);
+                        exit(2);
+                    }
+                }
                 if (entry->entries[3] >= 128)
 /* Needs fixing */
                     if ((c < 128) || (c == -1))
                         if ((entry->entries[1] < 128) ||
-                            (entry->entries[1] == bchar)) 
+                            (entry->entries[1] == bchar))
                             seven_bit_calculated = 0;
                 check_existence_and_safety(c, entry->entries[3],
                     ligature_commands[entry->entries[2]],
                     "%s character generated by (H %X)");
+                if (entry->entries[3] >0xffff) {
+                    fprintf(stderr, "%s character (H %X) generated by (H %X) "
+                        "exceeds ffff\n", ligature_commands[entry->entries[2]], entry->entries[3], c);
+                    exit(2);
+                }
             } else {
                 check_existence_and_safety(c, entry->entries[1],
                     "KRN", "%s character examined by (H %X)");
+                if (entry->entries[1] > 0xffff) {
+                    fprintf(stderr, "KRN character (H %X) examined by (H %X) "
+                        "exceeds ffff\n", entry->entries[1], c);
+                    exit(2);
+                }
             }
         }
         if (entry->entries[0] >= STOP_FLAG) lig_ptr = nl;
@@ -453,16 +498,18 @@ check_ligature_program(unsigned c, unsigned lab)
 void
 check_ligature_infinite_loops(void)
 {
-    list entry = hash_entries.front;
-    hash_list tt;
+    unsigned key;
 
-    while (entry != NULL) {
-        tt = (hash_list) entry->contents;
-        if (tt->new_class > LIG_SIMPLE)
-             l_f(tt, tt->x, tt->y);
-        entry = entry->ptr;
+    for (key = 0; key < PRIME; key++) {
+        hash_list tt = hash_table[key];
+        while (tt != NULL) {
+            if (tt->new_class > LIG_SIMPLE)
+                l_f(tt, tt->x, tt->y);
+            tt = tt->ptr;
+        }
     }
-    if (y_lig_cycle != 0x80000000) {
+
+    if (y_lig_cycle != CHAR_BOUNDARY) {
         if (x_lig_cycle == CHAR_BOUNDARY) {
             warning_1("Infinite ligature loop starting with boundary and %d",
                       y_lig_cycle);
@@ -471,7 +518,7 @@ check_ligature_infinite_loops(void)
                       x_lig_cycle, y_lig_cycle);
         }
         clear_ligature_entries();
-        nl = 0; bchar = CHAR_BOUNDARY; bchar_label = 0;
+        nl = 0; bchar = CHAR_BOUNDARY; bchar_label = MAX_LABEL;
     }
 }
 
@@ -483,7 +530,7 @@ doublecheck_ligatures(void)
     if (nl>0) {
         for (i=0; i<nl; i++) {
             if (lig_kern_table[i].entries[2] < KERN_FLAG) {
-                if (lig_kern_table[i].entries[0] != CHAR_BOUNDARY) {
+                if (lig_kern_table[i].entries[0] < 255) {
                     doublecheck_existence(
                         lig_kern_table[i].entries[1],
                         ligature_commands[lig_kern_table[i].entries[2]],
@@ -509,6 +556,42 @@ output_ofm_ligkern(void)
     unsigned i;
     four_entries *entry;
 
+    if (extra_loc_needed) {	/* lk_offset==1 */
+        if (ofm_level == OFM_TFM) {
+            out_ofm(255); out_ofm(bchar); out_ofm(0); out_ofm(0);
+        } else {
+            out_ofm_2(255); out_ofm_2(bchar); out_ofm_2(0); out_ofm_2(0);
+        }
+    } else {	/* output the redirection specs */
+        if (ofm_level == OFM_TFM) {
+            for (sort_ptr = 1; sort_ptr <= lk_offset; sort_ptr++) {
+                unsigned t = label_table[label_ptr].rr;
+                if (bchar != CHAR_BOUNDARY) {
+                    out_ofm(255); out_ofm(bchar);
+                } else {
+                    out_ofm(254); out_ofm(0);
+                }
+                out_ofm_2(t+lk_offset);
+                do {
+                    label_ptr--;
+                } while(label_table[label_ptr].rr == t);
+            }
+        } else {
+            for (sort_ptr = 1; sort_ptr <= lk_offset; sort_ptr++) {
+                unsigned t = label_table[label_ptr].rr;
+                if (bchar != CHAR_BOUNDARY) {
+                    out_ofm_2(255); out_ofm_2(bchar);
+                } else {
+                    out_ofm_2(254); out_ofm_2(0);
+                }
+                out_ofm_2((t+lk_offset) / 256); out_ofm_2((t+lk_offset) % 256);
+                do {
+                    label_ptr--;
+                } while(label_table[label_ptr].rr == t);
+            }
+        }
+    }
+
     if (ofm_level == OFM_TFM) {
         for (i=0; i<nl; i++) {
             entry = lig_kern_table+i;
@@ -539,6 +622,9 @@ retrieve_ligkern_table(unsigned char *ofm_lig_table,
     four_entries *entry;
     unsigned char *table_entry;
 
+    lig_kern_size = nl;
+    lig_kern_table = (four_entries *) xcalloc(lig_kern_size, sizeof(four_entries));
+
     if (ofm_level == OFM_TFM) {
         for (i=0; i<nl; i++) {
             entry = lig_kern_table+i;
@@ -562,6 +648,29 @@ retrieve_ligkern_table(unsigned char *ofm_lig_table,
                                 + ((*(table_entry+7)) & 0xff);
         }
     }
+
+    activity = (unsigned char *) xcalloc(lig_kern_size, sizeof(unsigned char));
+
+    if (nl > 0) {
+        if (lig_kern_table[0].entries[0] == 255) {
+            bchar = lig_kern_table[0].entries[1];
+            print_boundary_char(bchar);
+            activity[0] = A_PASS_THROUGH;
+        }
+        if (lig_kern_table[nl-1].entries[0] == 255) {
+            unsigned r = 256 * lig_kern_table[nl-1].entries[2] + lig_kern_table[nl-1].entries[3];
+            if (r >= nl) {
+                fprintf(stderr, "Ligature/kern starting index for boundarychar is too large;\n"
+                                "so I removed it.\n");
+                changed = TRUE;
+            } else {
+                bchar_label = r;
+                activity[r] = A_ACCESSIBLE;
+            }
+            activity[nl-1] = A_PASS_THROUGH;
+        }
+    }
+
     kern_table = (fix *) xmalloc((nk+1)*sizeof(int));
     for (i=0; i<nk; i++) {
         table_entry = ofm_kern_table+(4*i);
