@@ -172,7 +172,7 @@ getDesignSize(XeTeXFont font)
     return XeTeXFontMgr::GetFontManager()->getDesignSize(font);
 }
 
-const char*
+char*
 getFontFilename(XeTeXLayoutEngine engine, uint32_t* index)
 {
     return xstrdup(engine->font->getFilename(index));
@@ -1018,8 +1018,5 @@ usingOpenType(XeTeXLayoutEngine engine)
 bool
 isOpenTypeMathFont(XeTeXLayoutEngine engine)
 {
-    if (engine->font->getMathTable() != NULL)
-        return true;
-    else
-        return false;
+    return hb_ot_math_has_data(hb_font_get_face(engine->font->getHbFont()));
 }
