@@ -2,7 +2,7 @@
  Part of the XeTeX typesetting system
  Copyright (c) 1994-2008 by SIL International
  Copyright (c) 2009, 2011 by Jonathan Kew
- Copyright (c) 2012, 2013 by Khaled Hosny
+ Copyright (c) 2012-2015 by Khaled Hosny
  Copyright (c) 2012, 2013 by Jiang Jiang
 
  SIL Author(s): Jonathan Kew
@@ -88,6 +88,22 @@ authorization from the copyright holders.
 
 #include <assert.h>
 
+/* for reading input files, we don't need the default locking routines
+   as xetex is a single-threaded program */
+#ifdef WIN32
+#ifdef __MINGW32__
+/* MinGW (both 32- and 64-bit) has problems with _getc_nolock() and/or _ungetc_nolock() */
+#define GETC(f)      getc(f)
+#define UNGETC(c,f)  ungetc(c,f)
+#else
+#define GETC(f)      _getc_nolock(f)
+#define UNGETC(c,f)  _ungetc_nolock(c,f)
+#endif
+#else
+#define GETC(f)      getc_unlocked(f)
+#define UNGETC(c,f)  ungetc(c,f)
+#endif
+
 /* tables/values used in UTF-8 interpretation -
    code is based on ConvertUTF.[ch] sample code
    published by the Unicode consortium */
@@ -260,7 +276,7 @@ uclose(UFILE* f)
 }
 
 static void
-buffer_overflow()
+buffer_overflow(void)
 {
     fprintf (stderr, "! Unable to read an entire line---bufsize=%u.\n",
                              (unsigned) bufsize);
@@ -342,17 +358,17 @@ static uint32_t *utf32Buf = NULL;
             byteBuffer = (char*) xmalloc(bufsize + 1);
 
         /* Recognize either LF or CR as a line terminator; skip initial LF if prev line ended with CR.  */
-        i = getc(f->f);
+        i = GETC(f->f);
         if (f->skipNextLF) {
             f->skipNextLF = 0;
             if (i == '\n')
-                i = getc(f->f);
+                i = GETC(f->f);
         }
 
         if (i != EOF && i != '\n' && i != '\r')
             byteBuffer[bytesRead++] = i;
         if (i != EOF && i != '\n' && i != '\r')
-            while (bytesRead < bufsize && (i = getc(f->f)) != EOF && i != '\n' && i != '\r')
+            while (bytesRead < bufsize && (i = GETC(f->f)) != EOF && i != '\n' && i != '\r')
                 byteBuffer[bytesRead++] = i;
 
         if (i == EOF && errno != EINTR && bytesRead == 0)
@@ -468,7 +484,7 @@ static UBreakIterator* brkIter = NULL;
 static int brkLocaleStrNum = 0;
 
 void
-linebreakstart(int f, integer localeStrNum, const uint16_t* text, integer textLength)
+linebreakstart(int f, integer localeStrNum, uint16_t* text, integer textLength)
 {
     UErrorCode status = U_ZERO_ERROR;
     char* locale = (char*)gettexstring(localeStrNum);
@@ -513,7 +529,7 @@ linebreakstart(int f, integer localeStrNum, const uint16_t* text, integer textLe
 }
 
 int
-linebreaknext()
+linebreaknext(void)
 {
     if (brkIter != NULL)
         return ubrk_next((UBreakIterator*)brkIter);
@@ -609,7 +625,7 @@ load_mapping_file(const char* s, const char* e, char byteMapping)
         if (mapFile) {
             uint32_t mappingSize;
             Byte* mapping;
-            TECkit_Status status;
+            /* TECkit_Status status; */
             fseek(mapFile, 0, SEEK_END);
             mappingSize = ftell(mapFile);
             fseek(mapFile, 0, SEEK_SET);
@@ -617,12 +633,12 @@ load_mapping_file(const char* s, const char* e, char byteMapping)
             fread(mapping, 1, mappingSize, mapFile);
             fclose(mapFile);
             if (byteMapping != 0)
-                status = TECkit_CreateConverter(mapping, mappingSize,
+                /* status = */ TECkit_CreateConverter(mapping, mappingSize,
                                             false,
                                             UTF16_NATIVE, kForm_Bytes,
                                             &cnv);
             else
-                status = TECkit_CreateConverter(mapping, mappingSize,
+                /* status = */ TECkit_CreateConverter(mapping, mappingSize,
                                             true,
                                             UTF16_NATIVE, UTF16_NATIVE,
                                             &cnv);
@@ -643,7 +659,7 @@ load_mapping_file(const char* s, const char* e, char byteMapping)
 
 char *saved_mapping_name = NULL;
 void
-checkfortfmfontmapping()
+checkfortfmfontmapping(void)
 {
     char* cp = strstr((char*)nameoffile + 1, ":mapping=");
     if (saved_mapping_name != NULL) {
@@ -661,7 +677,7 @@ checkfortfmfontmapping()
 }
 
 void*
-loadtfmfontmapping()
+loadtfmfontmapping(void)
 {
     void* rval = NULL;
     if (saved_mapping_name != NULL) {
@@ -679,7 +695,8 @@ applytfmfontmapping(void* cnv, int c)
     UniChar in = c;
     Byte out[2];
     UInt32 inUsed, outUsed;
-    TECkit_Status status = TECkit_ConvertBuffer((TECkit_Converter)cnv,
+    /* TECkit_Status status; */
+    /* status = */ TECkit_ConvertBuffer((TECkit_Converter)cnv,
             (const Byte*)&in, sizeof(in), &inUsed, out, sizeof(out), &outUsed, 1);
     if (outUsed < 1)
         return 0;
@@ -883,9 +900,9 @@ readFeatureNumber(const char* s, const char* e, hb_tag_t* f, int* v)
 }
 
 static void*
-loadOTfont(PlatformFontRef fontRef, XeTeXFont font, Fixed scaled_size, const char* cp1)
+loadOTfont(PlatformFontRef fontRef, XeTeXFont font, Fixed scaled_size, char* cp1)
 {
-    XeTeXLayoutEngine engine;
+    XeTeXLayoutEngine engine = NULL;
     hb_tag_t script = HB_TAG_NONE;
     char * language = NULL;
     hb_feature_t* features = NULL;
@@ -893,8 +910,8 @@ loadOTfont(PlatformFontRef fontRef, XeTeXFont font, Fixed scaled_size, const cha
     int nFeatures = 0;
     int nShapers = 0;
 
-    const char* cp2;
-    const char* cp3;
+    char* cp2;
+    char* cp3;
 
     hb_tag_t tag;
 
@@ -911,10 +928,13 @@ loadOTfont(PlatformFontRef fontRef, XeTeXFont font, Fixed scaled_size, const cha
 
     if (reqEngine == 'O' || reqEngine == 'G') {
         shapers = (char**) xrealloc(shapers, (nShapers + 1) * sizeof(char *));
-        if (reqEngine == 'O')
-            shapers[nShapers] = "ot";
-        else if (reqEngine == 'G')
-            shapers[nShapers] = "graphite2";
+        if (reqEngine == 'O') {
+            static char ot_const[] = "ot";
+            shapers[nShapers] = ot_const;
+        } else if (reqEngine == 'G') {
+            static char graphite2_const[] = "graphite2";
+            shapers[nShapers] = graphite2_const;
+        }
         nShapers++;
     }
 
@@ -1568,12 +1588,12 @@ makefontdef(integer f)
     uint16_t flags = 0;
     uint32_t rgba;
     Fixed size;
-    const char* filename;
+    char* filename;
     uint32_t index;
     uint8_t filenameLen;
     int fontDefLength;
     char* cp;
-    PlatformFontRef fontRef = 0;
+    /* PlatformFontRef fontRef = 0; */
     float extend = 1.0;
     float slant = 0.0;
     float embolden = 0.0;
@@ -1617,7 +1637,7 @@ makefontdef(integer f)
         XeTeXLayoutEngine engine;
 
         engine = (XeTeXLayoutEngine)fontlayoutengine[f];
-        fontRef = getFontRef(engine);
+        /* fontRef = */ getFontRef(engine);
         filename = getFontFilename(engine, &index);
         assert(filename);
 
@@ -1719,7 +1739,7 @@ makefontdef(integer f)
 }
 
 int
-applymapping(void* pCnv, const uint16_t* txtPtr, int txtLen)
+applymapping(void* pCnv, uint16_t* txtPtr, int txtLen)
 {
     TECkit_Converter cnv = (TECkit_Converter)pCnv;
     UInt32 inUsed, outUsed;
@@ -1742,7 +1762,7 @@ retry:
 
     switch (status) {
         case kStatus_NoError:
-            txtPtr = (const UniChar*)mappedtext;
+            txtPtr = (UniChar*)mappedtext;
             return outUsed / sizeof(UniChar);
 
         case kStatus_OutputBufferFull:
@@ -1917,11 +1937,55 @@ get_native_glyph(void* pNode, unsigned index)
 }
 
 void
-store_justified_native_glyphs(void* node)
+store_justified_native_glyphs(void* pNode)
 {
-#ifdef XETEX_MAC /* this is only called for AAT fonts */
-    (void)DoAATLayout(node, 1);
+    memoryword* node = (memoryword*)pNode;
+    unsigned f = native_font(node);
+
+#ifdef XETEX_MAC /* separate Mac-only codepath for AAT fonts */
+    if (fontarea[f] == AAT_FONT_FLAG) {
+        (void)DoAATLayout(node, 1);
+        return;
+    }
 #endif
+
+    /* save desired width */
+    int savedWidth = node_width(node);
+
+    measure_native_node(node, 0);
+
+    if (node_width(node) != savedWidth) {
+        /* see how much adjustment is needed overall */
+        double justAmount = Fix2D(savedWidth - node_width(node));
+
+        /* apply justification to spaces (or if there are none, distribute it to all glyphs as a last resort) */
+        FixedPoint* locations = (FixedPoint*)native_glyph_info_ptr(node);
+        uint16_t* glyphIDs = (uint16_t*)(locations + native_glyph_count(node));
+        int glyphCount = native_glyph_count(node);
+        int spaceCount = 0, i;
+
+        int spaceGlyph = mapchartoglyph(f, ' ');
+        for (i = 0; i < glyphCount; ++i)
+            if (glyphIDs[i] == spaceGlyph)
+                spaceCount++;
+
+        if (spaceCount > 0) {
+            double adjustment = 0;
+            int spaceIndex = 0;
+            for (i = 0; i < glyphCount; ++i) {
+                locations[i].x = D2Fix(Fix2D(locations[i].x) + adjustment);
+                if (glyphIDs[i] == spaceGlyph) {
+                    spaceIndex++;
+                    adjustment = justAmount * spaceIndex / spaceCount;
+                }
+            }
+        } else {
+            for (i = 1; i < glyphCount; ++i)
+                locations[i].x = D2Fix(Fix2D(locations[i].x) + justAmount * i / (glyphCount - 1));
+        }
+
+        node_width(node) = savedWidth;
+    }
 }
 
 void
@@ -1944,9 +2008,9 @@ measure_native_node(void* pNode, int use_glyph_metrics)
 
         XeTeXLayoutEngine engine = (XeTeXLayoutEngine)(fontlayoutengine[f]);
 
-        FixedPoint* locations;
+        FixedPoint* locations = NULL;
         uint16_t* glyphIDs;
-        Fixed* glyphAdvances;
+        Fixed* glyphAdvances = NULL;
         int totalGlyphCount = 0;
 
         /* need to find direction runs within the text, and call layoutChars separately for each */
@@ -2044,10 +2108,10 @@ measure_native_node(void* pNode, int use_glyph_metrics)
                     locations[i].x = D2Fix(positions[i].x);
                     locations[i].y = D2Fix(positions[i].y);
                 }
-                width = D2Fix(positions[totalGlyphCount].x);
+                width = positions[totalGlyphCount].x;
             }
 
-            node_width(node) = width;
+            node_width(node) = D2Fix(width);
             native_glyph_count(node) = totalGlyphCount;
             native_glyph_info_ptr(node) = glyph_info;
 
@@ -2498,7 +2562,7 @@ aatprintfontname(int what, CFDictionaryRef attributes, int param1, int param2)
 void
 printglyphname(integer font, integer gid)
 {
-    char* s;
+    const char* s;
     int len = 0;
 #ifdef XETEX_MAC
     if (fontarea[font] == AAT_FONT_FLAG) {
@@ -2507,7 +2571,7 @@ printglyphname(integer font, integer gid)
 #endif
     if (fontarea[font] == OTGR_FONT_FLAG) {
         XeTeXLayoutEngine engine = (XeTeXLayoutEngine)fontlayoutengine[font];
-        s = (char*)getGlyphName(getFont(engine), gid, &len);
+        s = getGlyphName(getFont(engine), gid, &len);
     } else {
         fprintf(stderr, "\n! Internal error: bad native font flag in `print_glyph_name'\n");
         exit(3);
@@ -2530,8 +2594,8 @@ u_open_in(unicodefile* f, integer filefmt, const_string fopen_mode, integer mode
         int B1, B2;
         if (mode == AUTO) {
             /* sniff encoding form */
-            B1 = getc((*f)->f);
-            B2 = getc((*f)->f);
+            B1 = GETC((*f)->f);
+            B2 = GETC((*f)->f);
             if (B1 == 0xfe && B2 == 0xff)
                 mode = UTF16BE;
             else if (B2 == 0xfe && B1 == 0xff)
@@ -2543,7 +2607,7 @@ u_open_in(unicodefile* f, integer filefmt, const_string fopen_mode, integer mode
                 mode = UTF16LE;
                 rewind((*f)->f);
             } else if (B1 == 0xef && B2 == 0xbb) {
-                int B3 = getc((*f)->f);
+                int B3 = GETC((*f)->f);
                 if (B3 == 0xbf)
                     mode = UTF8;
             }
@@ -2573,12 +2637,18 @@ open_dvi_output(FILE** fptr)
         return open_output(fptr, FOPEN_WBIN_MODE);
     } else {
         const char *p = (const char*)nameoffile+1;
-        char    *cmd, *q;
+        char    *cmd, *q, *bindir = NULL;
         int len = strlen(p);
         while (*p)
             if (*p++ == '\"')
                 ++len;
         len += strlen(outputdriver);
+#ifndef WIN32
+        if (!kpse_absolute_p(outputdriver, true))
+            bindir = kpse_var_value("SELFAUTOLOC");
+        if (bindir)
+            len += strlen(bindir) + 1;
+#endif
         if (output_directory)
             len += strlen(output_directory);
         len += 10; /* space for -o flag, quotes, NUL */
@@ -2586,7 +2656,17 @@ open_dvi_output(FILE** fptr)
             if (*p == '\"')
                 ++len;  /* allow extra space to escape quotes in filename */
         cmd = xmalloc(len);
+#ifdef WIN32
         strcpy(cmd, outputdriver);
+#else
+        if (bindir) {
+            strcpy(cmd, bindir);
+            strcat(cmd, "/");
+            strcat(cmd, outputdriver);
+        } else {
+            strcpy(cmd, outputdriver);
+        }
+#endif
         strcat(cmd, " -o \"");
         if (output_directory) {
             len = strlen(output_directory);
@@ -2619,7 +2699,7 @@ open_dvi_output(FILE** fptr)
 #if defined(WIN32)
         {
             wchar_t *tmp1w;
-            char *p, *pp, *bindir, *fullcmd, *prgnam;
+            char *p, *pp, *fullcmd, *prgnam;
             bindir = kpse_var_value("SELFAUTOLOC");
             for(pp = bindir; *pp; pp++) {
                 if(*pp == '/') *pp = '\\';
@@ -2680,17 +2760,17 @@ get_uni_c(UFILE* f)
 
     switch (f->encodingMode) {
         case UTF8:
-            c = rval = getc(f->f);
+            c = rval = GETC(f->f);
             if (rval != EOF) {
                 uint16_t extraBytes = bytesFromUTF8[rval];
                 switch (extraBytes) {   /* note: code falls through cases! */
-                    case 3: c = getc(f->f);
+                    case 3: c = GETC(f->f);
                         if (c < 0x80 || c >= 0xc0) goto bad_utf8;
                         rval <<= 6; rval += c;
-                    case 2: c = getc(f->f);
+                    case 2: c = GETC(f->f);
                         if (c < 0x80 || c >= 0xc0) goto bad_utf8;
                         rval <<= 6; rval += c;
-                    case 1: c = getc(f->f);
+                    case 1: c = GETC(f->f);
                         if (c < 0x80 || c >= 0xc0) goto bad_utf8;
                         rval <<= 6; rval += c;
                     case 0:
@@ -2698,7 +2778,7 @@ get_uni_c(UFILE* f)
 
                     bad_utf8:
                         if (c != EOF)
-                            ungetc(c, f->f);
+                            UNGETC(c, f->f);
                     case 5:
                     case 4:
                         badutf8warning();
@@ -2709,14 +2789,14 @@ get_uni_c(UFILE* f)
             break;
 
         case UTF16BE:
-            rval = getc(f->f);
+            rval = GETC(f->f);
             if (rval != EOF) {
                 rval <<= 8;
-                rval += getc(f->f);
+                rval += GETC(f->f);
                 if (rval >= 0xd800 && rval <= 0xdbff) {
-                    int lo = getc(f->f);
+                    int lo = GETC(f->f);
                     lo <<= 8;
-                    lo += getc(f->f);
+                    lo += GETC(f->f);
                     if (lo >= 0xdc00 && lo <= 0xdfff)
                         rval = 0x10000 + (rval - 0xd800) * 0x400 + (lo - 0xdc00);
                     else {
@@ -2729,12 +2809,12 @@ get_uni_c(UFILE* f)
             break;
 
         case UTF16LE:
-            rval = getc(f->f);
+            rval = GETC(f->f);
             if (rval != EOF) {
-                rval += (getc(f->f) << 8);
+                rval += (GETC(f->f) << 8);
                 if (rval >= 0xd800 && rval <= 0xdbff) {
-                    int lo = getc(f->f);
-                    lo += (getc(f->f) << 8);
+                    int lo = GETC(f->f);
+                    lo += (GETC(f->f) << 8);
                     if (lo >= 0xdc00 && lo <= 0xdfff)
                         rval = 0x10000 + (rval - 0xd800) * 0x400 + (lo - 0xdc00);
                     else {
@@ -2773,7 +2853,7 @@ get_uni_c(UFILE* f)
 #endif
 
         case RAW:
-            rval = getc(f->f);
+            rval = GETC(f->f);
             break;
 
         default:
@@ -2786,7 +2866,7 @@ get_uni_c(UFILE* f)
 }
 
 void
-makeutf16name()
+makeutf16name(void)
 {
     unsigned char* s = nameoffile + 1;
     uint32_t rval;
